
#ifndef INC_DAA
#define INC_DAA

#include "resource.h"       // main symbols
#include "GlobalParams.h"
#include "DrawUtils.h"
#include "UtilityFunctions.h"

/////////////////////////////////////////////////////////////////////////////
// Daa.h : main header file for the DAA application
/////////////////////////////////////////////////////////////////////////////
#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

/////////////////////////////////////////////////////////////////////////////
// DAA CONSTANTS and ARRAYS
/////////////////////////////////////////////////////////////////////////////

// --------------------------------------------- audio buffers
const int N_BUFIN = 16;				
const int IN_BUF_LEN = 1024; 

// --------------------------------------------- must be a multiple of IN_BUF_LEN
// --------------------------------------------- 192000(approx) * 2 ch. * 32 seconds = 50 mega byte
const int IN_CBUF_LEN = 192 * IN_BUF_LEN * 2 * 32; 
const int N_BUFOUT = 32;		
const int OUT_BUF_LEN = 2048;

// Audio Device - Realtek(computer LAB) - Precision 
//      10%: 4000 6400 6800 7000 99000
//  1/10000: 8000 10000 11025 13600 20400 22050 25000 35000 44100
// 2/100000: 5000 20000
// 1/100000: 6000 9000 12000 12800 15000 18000 21000 24000 27000 30000 33000 45000 90000 96000

// ===============================================
//  Normal sampling: 44100 / 96000 / 192000
// ===============================================
// ------------------------------------------------------------------ SAMPLES_PER_SEC = 96000
//const int SAMPLES_PER_SEC = 96000;
//const int MAX_FHT_SAMPLES = 32768;
//const int PULSE_FHT_SAMPLES = 16384;
//const int FASTSWEEP_FHT_SAMPLES = 32768;
// ------------------------------------------------------------------ SAMPLES_PER_SEC = 192000
const int SAMPLES_PER_SEC = 192000;
const int MAX_FHT_SAMPLES = 65536;
const int PULSE_FHT_SAMPLES = 16384;
const int FASTSWEEP_FHT_SAMPLES = 65536;

// ------------------------------------------------------------------
const int WTAB_LEN = SAMPLES_PER_SEC;

// ------------------------------------------------------------------
const int PULSE_TOTAL_SAMPLES = int(SAMPLES_PER_SEC * 0.4);
const int PULSE_SAMPLES = int(SAMPLES_PER_SEC * 0.2); 
const int FASTSWEEP_SAMPLES = int(SAMPLES_PER_SEC * 0.4);
//
const double kdb = 8.685889638; 
const double PI = 3.1415926535;

const double Root2f2 = 0.707106781187;
const double Root2   = 1.414213562373;
const double Root2x2 = 2.828427124746;

const WORD PRESSED_NOW = 0x8000;
const int RESP_TAB_LEN = 9000; // more than the effective display x pixels 

const int out_pulse_millisec = (PULSE_SAMPLES * 1000) / SAMPLES_PER_SEC ;

// ------------------------------------------------------------------ Input output buffers
static __int16 pbufIn[N_BUFIN][IN_BUF_LEN];
static float pCbufIn[IN_CBUF_LEN];

static __int16 pbufOut[N_BUFOUT][OUT_BUF_LEN];
static float wtab[WTAB_LEN];

static HWAVEOUT hwOut;
static HWAVEIN  hwIn;
static WAVEHDR  pwhOut[N_BUFOUT];
static WAVEHDR  pwhIn[N_BUFIN];

static float calibrateTab[RESP_TAB_LEN];
static float responseTab[RESP_TAB_LEN];

// ------------------------------------------------------------------ FHT
const int samplesMax = MAX_FHT_SAMPLES;
static float senTab[samplesMax];
static float cosTab[samplesMax];
static float aTab[samplesMax + 1];
static float bTab[samplesMax + 1];
static float hTab[samplesMax];
static int permTab[samplesMax];

// ------------------------------------------------------------------ 
const int WAVETYPE_SINE   = 0; 
const int WAVETYPE_TRIANG = 1; 
const int WAVETYPE_RAMPp  = 2; 
const int WAVETYPE_RAMPm  = 3; 
const int WAVETYPE_SQUARE = 4; 
const int WAVETYPE_NOISE  = 5;
//
const int ANTYPE_WAVE     = 0; 
const int ANTYPE_BURST    = 1; 
const int ANTYPE_SPECTR   = 2; 
const int ANTYPE_SWEEP    = 3; 
const int ANTYPE_FASTSW   = 4; 
const int ANTYPE_PULSE    = 5; 
//
const int REGTYPE_FREQ    = 0;
const int REGTYPE_FMAX    = 1;
const int REGTYPE_FMIN    = 2;
const int REGTYPE_DBMAX   = 3;
const int REGTYPE_DBMIN   = 4;
const int REGTYPE_SLOT    = 5;
const int REGTYPE_CALIB   = 6;
//
const int REGTYPE_KB2_FIRST = 7;
const int REGTYPE_DUTY    = 7;
const int REGTYPE_BURST   = 8;
const int REGTYPE_PAUSE   = 9;
const int REGTYPE_BEGIN   = 10;
const int REGTYPE_END     = 11;
const int REGTYPE_TIME    = 12;
const int REGTYPE_OUTLEV  = 13;

// ------------------------------------------------------------------
extern COLORREF black, panels, wavepanel; 
extern COLORREF col_pressed, col_lighted_border; 
extern COLORREF grey0, grey1, grey2, grey3, grey4, grey5, grey6, grey7, grey8, grey9, white;
extern COLORREF dgrid, dback, trace;

extern CBrush backBrush;

// ------------------------------------------------------------------ display width and height
extern int dispdx;
extern int dispdy;

// ------------------------------------------------------------------ spectrum grid fontsize, width and height
extern int fontsize;
extern int spectrumGridFontSize;
extern int leftborder; 
extern int downborder;
extern int dispx;
extern int dispy;

// ------------------------------------------------------------------
extern int sweepLimit;
extern int sweepPos;
extern int sweepSample;
extern double sweepFreq;
extern int sweepStatus;
extern int random;
extern double random2;

extern int cbufPos;

extern double delaypos, deltapos;

extern int pulse_position;

extern bool calibratedResponse;

extern bool DisplaySamples;
extern bool multiDraw;
extern bool spectrum3D;
extern bool burstflag;

extern int BufOutIndex, BufInIndex;
extern int activeOutBuffers;
extern int activeInBuffers;

extern int numCH;

extern double phase; 
extern int burstcounter;
extern int readpos;
extern int readpos1, readpos2;
extern int sampPerScreen;
extern int pulseCbuffPos;
extern double tglev;

extern double trigvoid, trigpos, triglev, trigist, trigstab, trigstop;
extern BOOL trigslope;
extern BOOL trigburst;
extern int trigsample;
extern bool triggered;

extern double mVdc, mVdcInst;
extern double mVpp, mVppInst;
extern double mVpp1, mVpp2;

extern bool Calibrating;
extern double calib_vmax1, calib_vmin1, calib_vmax2, calib_vmin2;

extern double vmax1, vmin1, vmax2, vmin2;
extern double vmax_sweep, vmin_sweep;
extern double vmidinst1, vmidinst2;
extern double vmid1, vmid2;
extern double rmsSum1, rmsSum2, rmsSamplesCounter;
extern double vrms1, vrms2;

extern double oldfreq;

extern double gridLight, osdIntensity, traceIntensity;

extern double scaleX, scaleY1, scaleY2;
extern double regX, regY1, regY2, filterFrequency1, filterFrequency2;
extern double posY1, posY2;
extern double posY1_discrete, posY2_discrete;
extern int calY1, calY2;

extern double gain1;
extern double gain2;

extern double curs1, curs2;
extern double responsespeed;
extern int curstype;
extern int timetype, volttype;
extern int xlogtype, ylogtype;

extern int inputDevice;
extern int outputDevice;
extern int inputState;
extern int outputState;

extern int selectCH, trigCH;
extern BOOL ac1, ac2, neg1, neg2;
extern double mul1, mul2;

extern int wavetype;
extern int analysisType;
extern int regulationType;
extern int trigType;

extern double freq;
extern double freqMin;
extern double freqMax;
extern double sweepTime;
extern double start_sampling_ms;
extern double sampling_ms;

extern double duty;
extern double burstCycles;
extern double pauseCycles;
extern double dbmin;
extern double dbmax;
extern double outlevel;
extern double deltalevel1;
extern double deltalevel2;

extern int    bandwidth;
extern double band;

extern int    fhtwindow;

extern int	  Filter_CH1;
extern int	  Filter_CH2;
extern int	  Filters_Npoles;
extern void	  DAA_Filters_Init();

extern double fmeterTrigDown;
extern double fmeterTrigUp;
extern bool	  fmeterFlag;                                                                
extern int	  fmeterSamples;             
extern int	  fmeterCycles;    
extern int	  fmeterResultSamples;
extern int	  fmeterResultCycles; 

extern int	  initSamples;
extern int	  fht_samples;

extern bool	  pulseFlag;
extern bool	  referenceFlag;


/////////////////////////////////////////////////////////////////////////////
// CDaaApp:
// See Daa.cpp for the implementation of this class
//

class CDaaApp : public CWinApp
{
public:
	CDaaApp();

	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDaaApp)
	public:
	virtual BOOL InitInstance();
	virtual BOOL ExitInstance();
	virtual BOOL OnIdle(LONG lCount);
	//}}AFX_VIRTUAL

	//
	bool DaaFileOpen();
	void DaaMRUFileOpen(UINT i);

	void AddInputDevicesMenu();
	void AddOutputDevicesMenu();


// Implementation

	//{{AFX_MSG(CDaaApp)
	afx_msg void OnAppAbout();
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////



/*================================================================================
===== CText ==================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CText : public CObject
{
protected:
    int x, y;       
    int font_xsize;
	int font_ysize;
	int font_weight;
    CString fontname;    
    COLORREF color;
    COLORREF shadow;
	bool transparent;
    CString text;

public:

    CText() {};
   
    ~CText() {}

    CText (const CText &t)
    {
        x = t.x;
        y = t.y;
        font_xsize = t.font_xsize;
		font_ysize = t.font_ysize;
		font_weight = t.font_weight;
        fontname = t.fontname;
        color = t.color;
        shadow = t.shadow;
		transparent = t.transparent;
        text = t.text;
    }
        
    CText &  operator = (const CText &t)
    {
        x = t.x;
        y = t.y;
        font_xsize = t.font_xsize;
		font_ysize = t.font_ysize;
		font_weight = t.font_weight;
		fontname = t.fontname;
        color = t.color;
        shadow = t.shadow;
		transparent = t.transparent;
        text = t.text;
        return *this;
    } 


    void Init(int _x, int _y, 
					int xsize, int ysize, int weight,
					CString fname, COLORREF _color, COLORREF _shadow, bool _transparent)
    {
        x = _x;
        y = _y;
        font_xsize = xsize;
		font_ysize = ysize;
		font_weight = weight;
        fontname = fname;
        color = _color;
        shadow = _shadow;
		transparent = _transparent;
    }

	void InitColors(COLORREF _color, COLORREF _shadow, bool _transparent)
    {
        color = _color;
        shadow = _shadow;
		transparent = _transparent;
    }

    void Set(CString _text)
    {
		text = _text;
    }

	void Draw(CDC* pDC)
	{
		if (text != "")
		{
			CFont font;
			font.CreateFont(font_ysize, font_xsize, 0, 0, font_weight, 0, 0, 0, 0, 0, 0, CLEARTYPE_QUALITY, 0, fontname);

			CFont* pOldFont = pDC->SelectObject(&font);

			if (transparent)
			{
				pDC->SetBkMode (TRANSPARENT);
			}
			else
			{
				pDC->SetBkMode (OPAQUE);
				pDC->SetBkColor (pDC->GetPixel(x+8,y));
			}

			if (shadow != color)
			{
				pDC->SetTextColor(shadow);
				pDC->TextOut(x + 1, y + 1, text);
			}
			pDC->SetTextColor(color);

			pDC->TextOut(x, y, text);
			pDC->SelectObject(pOldFont);
		}
	}
};


/*================================================================================
===== CFrame ==================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CFrame : public CObject
{
public:
    int x, y;         // Position
    int w, h;         // Width and height
    int rad;          // Corners radius

    COLORREF border;
    COLORREF color;

    CText title;

    CFrame() {};
   
    ~CFrame() {}

    void    Init(int _x, int _y, int _w, int _h, int _rad, COLORREF _border, COLORREF _color)
    {
        x = _x;
        y = _y;
        w = _w;
        h = _h;
        rad = _rad;
        border = _border;
        color = _color;
    }

    
    void    InitTitle(int _tx, int _ty, 
					  int xsize, int ysize, int weight, CString fname, 
                      COLORREF color, COLORREF shadow, bool transparent,
                      CString _text)
    {
        title.Init( x + _tx, y + _ty, xsize, ysize, weight, fname, color, shadow, transparent);
        title.Set(_text);
    }

	void    SetText(CDC* pDC, CString _text)
	{
		 title.Set(_text);
		 Draw(pDC);
	}
    
	void	Draw(CDC* pDC)
	{
		if (w && h)
		{
			CPen pen(PS_SOLID, 1, border);
			CPen* pOldPen = pDC->SelectObject(&pen);
			CBrush brush(color);
			CBrush* pOldBrush = pDC->SelectObject(&brush);
			pDC->RoundRect(x, y, x + w, y + h, rad, rad);
			pDC->SelectObject(pOldPen);
			pDC->SelectObject(pOldBrush);
		}
		title.Draw(pDC);
	}
};

/*================================================================================
===== CLed ====================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CLed : public CObject 
{
protected:
    int x, y, w, h;     // Position and size
    int b;              // Border size
    int rx, ry, rw, rh; // Reflex position and size
    
         
    COLORREF offCol;
    COLORREF firstCol;
    COLORREF secondCol;
    COLORREF borderCol;  
    COLORREF reflexCol;       
   
    CText title;

    BOOL dcCreated;
    CDC dcLed;
    CBitmap bmpLed;

    
public:
    BOOL square;
    int color;

    CLed()
    {  
        square = FALSE;
        dcCreated = FALSE;
        color = 0;
        rw = rh = 0;
    }

    ~CLed() {}

    
    CLed (const CLed &_)
    {
        x = _.x;
        y = _.y;
    }
        
    CLed &  operator = (const CLed &_)
    {
        x = _.x;
        y = _.y;
        return *this;
    } 

	BOOL  Inside(CPoint point)
    {
        if (point.x < x || point.x > x + w ||
            point.y < y || point.y > y + h) return FALSE;
        return TRUE;
    }
           
    void    Init(int _x, int _y, int _w, int _h, int _b)
    {
        x = _x;
        y = _y;
        w = _w + _b;
        h = _h + _b;
        b = _b;
    }
 

    void    InitText(CString text, int tx, int ty,
					 int xsize, int ysize, int weight, 
                     COLORREF color, COLORREF shadow, bool transparent, CString fname)
    {
        title.Init( x + tx, y + ty, xsize, ysize, weight, fname, color, shadow, transparent);
        title.Set(text);
    }

    void    InitColors(COLORREF _offCol,
                       COLORREF _firstCol,
                       COLORREF _secondCol,
                       COLORREF _borderCol,
                       COLORREF _reflexCol)
    {
        offCol = _offCol;
        firstCol = _firstCol;
        secondCol = _secondCol;
        borderCol = _borderCol;
        reflexCol = _reflexCol;
    }

    void    InitReflex(int _rx, int _ry, int _rw, int _rh)
    {
        rx = _rx;
        ry = _ry;
        rw = _rw;
        rh = _rh;
    }
           
    void    Draw(CDC* pDC)
    {
        if (!dcCreated)
        {
            bmpLed.CreateCompatibleBitmap(pDC, w+2, h+2);
            dcLed.CreateCompatibleDC(pDC);
            dcLed.SelectObject(bmpLed);
            dcCreated = TRUE;
            dcLed.BitBlt(0, 0, w+2, h+2, pDC, x, y, SRCCOPY);
        }

        title.Draw(pDC);
        Update(pDC, -1);
    }

    void    Update(CDC* pDC, BOOL _color)
    {
        if (_color == color && color >= 0) return;   
        color = _color;

        COLORREF ledcol;
        if (color == 1) ledcol = firstCol;
        else if (color == 2) ledcol = secondCol;
		else if (color == 3) ledcol = RGB(255, 255, 0);
        else ledcol = offCol;
        CPen pen(PS_SOLID, 0, borderCol);
        CBrush brush(borderCol);
        CPen pen2(PS_SOLID, 0, ledcol);
        CBrush brush2(ledcol);

        COLORREF rcol;
        if (color > 0) rcol = RGB(255,255,255);
        else rcol = reflexCol;
        CPen pen3(PS_SOLID, 0, rcol);
        CBrush brush3(rcol);


        CPen* pOldPen = dcLed.SelectObject(&pen);
        CBrush* pOldBrush = dcLed.SelectObject(&brush);
        if (square) dcLed.RoundRect(0, 0, w, h, 4, 4);
        else dcLed.Ellipse(0, 0, w, h);
         
        
        dcLed.SelectObject(&pen2);
        dcLed.SelectObject(&brush2);
        if (square) dcLed.RoundRect(b, b, w - b, h - b, 5, 5);
        else dcLed.Ellipse(b, b, w - b, h - b);
        

        if (rw)
        {
            dcLed.SelectObject(&pen3);
            dcLed.SelectObject(&brush3);
            dcLed.Ellipse(rx - rw, ry - rh, rx + rw, ry + rh);
        }

        dcLed.SelectObject(pOldPen);
        dcLed.SelectObject(pOldBrush);

        pDC->BitBlt(x, y, w+2, h+2, &dcLed, 0, 0, SRCCOPY);
    }
};


/*================================================================================
===== CDisplay ================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CDisplay : public CObject 
{
protected:
    int x, y;
    int b, bUp, bIn;
    int dx, dy, ddx, ddy;
    int tx, ty;
    CText title;

	double px;
	float ky1, ky2;
	int py, py1, py2;
    BOOL dcCreated;
    
    CPen pens[32];
    CPen penblack;
    CPen pen3Dback;
 
public:
    CBitmap bmpGrid, bmpScreen;
    CDC dcGrid, dcScreen;
    BOOL pressed;
  
    CDisplay()
    {

		InitPositionAndSize();

        // Title
        title.Init(x + 30, y + 2, 0, 23, 0, "Arial black", RGB(50,50,50), RGB(230,230,230), true);
        title.Set("DAA V4.2        Digital audio analyser");
        
        penblack.CreatePen(PS_SOLID, 1, RGB(0,0,0));

        pens[0].CreatePen(PS_SOLID, 1, RGB(40,40,40));//RGB(0,70,0));
        pens[1].CreatePen(PS_SOLID, 1, RGB(0,90,0));
        pens[2].CreatePen(PS_SOLID, 1, RGB(0,110,0));
        pens[3].CreatePen(PS_SOLID, 1, RGB(0,160,0));
        pens[4].CreatePen(PS_SOLID, 1, RGB(60,200,0));
        pens[5].CreatePen(PS_SOLID, 1, RGB(100,250,0));
        pens[6].CreatePen(PS_SOLID, 1, RGB(120,255,0));
        pens[7].CreatePen(PS_SOLID, 1, RGB(140,255,0));
        pens[8].CreatePen(PS_SOLID, 1, RGB(160,255,0));
        pens[9].CreatePen(PS_SOLID, 1, RGB(180,255,0));
        pens[10].CreatePen(PS_SOLID, 1, RGB(200,255,0));
        pens[11].CreatePen(PS_SOLID, 1, RGB(210,255,0));
        pens[12].CreatePen(PS_SOLID, 1, RGB(220,255,0));
        pens[13].CreatePen(PS_SOLID, 1, RGB(230,255,0));
        pens[14].CreatePen(PS_SOLID, 1, RGB(240,255,0));
        pens[15].CreatePen(PS_SOLID, 1, RGB(250,255,0));
        pens[16].CreatePen(PS_SOLID, 1, RGB(255,255,0));
        pens[17].CreatePen(PS_SOLID, 1, RGB(255,255,0));
        pens[18].CreatePen(PS_SOLID, 1, RGB(255,255,0));
        pens[19].CreatePen(PS_SOLID, 1, RGB(255,250,0));        
        pens[20].CreatePen(PS_SOLID, 1, RGB(255,240,0));
        pens[21].CreatePen(PS_SOLID, 1, RGB(255,230,0));
        pens[22].CreatePen(PS_SOLID, 1, RGB(255,220,0));
        pens[23].CreatePen(PS_SOLID, 1, RGB(255,210,0));
        pens[24].CreatePen(PS_SOLID, 1, RGB(255,200,0));
        pens[25].CreatePen(PS_SOLID, 1, RGB(255,180,0));
        pens[26].CreatePen(PS_SOLID, 1, RGB(255,160,0));
        pens[27].CreatePen(PS_SOLID, 1, RGB(255,100,0));
        pens[28].CreatePen(PS_SOLID, 1, RGB(255,80,0));
        pens[29].CreatePen(PS_SOLID, 1, RGB(255,60,0));
        pens[30].CreatePen(PS_SOLID, 1, RGB(255,40,0));
        pens[31].CreatePen(PS_SOLID, 1, RGB(255,0,0));
    }

    ~CDisplay() 
	{
	}
   
    void    Draw(CDC* pDC);
    void    CreateGrid();
    void    Clear();
    void    DisplayAll(CDC* pDC);
    void    PlotSweepWave(CDC* pDC);
    void    PlotWave(CDC* pDC);
	void    DisplayReadout(CDC* pDC);
    void    PlotSpectrum3D(CDC* pDC);

	void  InitPositionAndSize()
	{
		dcCreated = FALSE;
		// ----------------------------------- Display upper left position
		x = 76;
		y = 9;
		// ---------------------------------------------------------- Lateral, superior and internal borders
		b = 16;
		bUp = 30;
		bIn = 7;
		// ---------------------------------------------------------- Display area, width and height
		dispdx = CLIENT_X - 438;
		dispdy = CLIENT_Y - 289;
		// ---------------------------------------------------------- FONTSIZE for READOUT
        fontsize = dispdx / 26;
		if (dispdy / 12 < fontsize) fontsize = int(dispdy / 12);
		if (fontsize > 32) fontsize = 32;
		if (fontsize < 16) fontsize = 16;
		// ---------------------------------------------------------- FONTSIZE for SPECTRUM and SPECTRUM-3D scale
        spectrumGridFontSize = int(dispdy / 28);
		if (dispdx / 36 > spectrumGridFontSize) spectrumGridFontSize = int(dispdx / 36);
		if (spectrumGridFontSize > 25) spectrumGridFontSize = 25;
		if (spectrumGridFontSize > dispdy / 18) spectrumGridFontSize = int(dispdy / 18);
        // ---------------------------------------------------------- Display area for SPECTRUM
		leftborder = int(spectrumGridFontSize * 2.8); 
		downborder = int(spectrumGridFontSize * 1.5);
		dispx = dispdx - leftborder;
		dispy = dispdy - downborder;
		// ----------------------------------------------------------
		ddx = dispdx;
		ddy = dispdy;
		dx = x + b + bIn;
		dy = y + bUp + bIn;
	}

	BOOL  Inside(CPoint point)
    {
		point.x -= b + bIn;
		point.y -= bUp + bIn;
        if (point.x < x || point.x > x + ddx ||
            point.y < y || point.y > y + ddy) return FALSE;
        return TRUE;
    }

    BOOL  ClickInside(CPoint point)
    {
		point.x -= b + bIn;
		point.y -= bUp + bIn;
        if (point.x < x || point.x > x + ddx ||
            point.y < y || point.y > y + ddy) return FALSE;
        pressed = TRUE;
        return TRUE;
    }

	float NormX(CPoint point)
	{
		point.x -= x + b + bIn;
		return float(point.x) / ddx;
	}

	float NormY(CPoint point)
	{
		point.y -= y + bUp + bIn;
		return 1 - float(point.y) / ddy;
	}

	float NormX_Spectrum(CPoint point)
	{
		point.x -= x + b + bIn + leftborder;
		return float(point.x) / (ddx - leftborder);
	}

	float NormY_Spectrum(CPoint point)
	{
		point.y -= y + bUp + bIn;
		return 1 - float(point.y) / (ddy - downborder);
	}

	int GetBorderUp()
	{
		return bUp + b;
	}

	//CBitmap GetBmpScreen()
	//{
	//	return bmpScreen;
	//}
};



/*================================================================================
===== CKey ====================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CKey : public CObject 
{
protected:
    int x, y;           // Position
    int w, h;           // Width and height
    int b;              // Border dimension
    int type;           // 0 = bistable / 1 = interlock / 2 = pulse
       

    COLORREF frameCol;      //  external rectangle
    COLORREF upBordCol;     //  key released external rectangle border
    COLORREF downBordCol;   //  key pressed external rectangle border
    
    COLORREF keyCol;        //  key face
    COLORREF lightedKeyCol; //  lighted key face
    COLORREF keyColBorder;  //  key face border
    
    COLORREF upCol1;        //  key released left-up border 
    COLORREF upCol2;        //  key released right-down border
    COLORREF downCol1;      //  key pressed left-up border
    COLORREF downCol2;      //  key pressed right-down border

    CText title;
    
public:
    BOOL pressed;
    BOOL lighted;

    CKey()
    {  
        pressed = FALSE;
        lighted = FALSE;
    }

    ~CKey() {}

    
    CKey (const CKey &k)
    {
        x = k.x;
        y = k.y;
    }
        
    CKey &  operator = (const CKey &k)
    {
        x = k.x;
        y = k.y;
        return *this;
    } 
       
    BOOL  Inside(CPoint point)
    {
        if (point.x < x || point.x > x + w ||
            point.y < y || point.y > y + h) return FALSE;
        return TRUE;
    }

    BOOL  ClickInside(CDC* pDC, CPoint point)
    {
        if (point.x < x || point.x > x + w ||
            point.y < y || point.y > y + h) return FALSE;
        if (type == 0) pressed = !pressed;
        else pressed = TRUE;
        Draw(pDC);
        return TRUE;
    }
    
    void    Init(int _x, int _y, int _w, int _h, int _b, int _type)
    {
        x = _x;
        y = _y;
        w = _w + _b * 2;
        h = _h + _b * 2;
        b = _b;
        type = _type;
    }

    void    InitText(CString text, int tx, int ty,
                     int xsize, int ysize, int weight,
					 COLORREF color, COLORREF shadow, bool transparent, CString fname)
    {
        title.Init( x + tx, y + ty, xsize, ysize, weight, fname, color, shadow, transparent);
        title.Set(text);
    }

    void    InitColors(COLORREF _frameCol,
                       COLORREF _upBordCol,
                       COLORREF _downBordCol,
                       COLORREF _keyCol,
                       COLORREF _keyColBorder,
                       COLORREF _lightedKeyCol,
                       COLORREF _upCol1, COLORREF _upCol2,
                       COLORREF _downCol1, COLORREF _downCol2)
    {
		frameCol = _frameCol;
        upBordCol = _upBordCol;
        downBordCol = _downBordCol;
        keyCol = _keyCol;
        lightedKeyCol = _lightedKeyCol;
        keyColBorder = _keyColBorder;
        upCol1 = _upCol1;
        upCol2 = _upCol2;
        downCol1 = _downCol1;
        downCol2 = _downCol2;
    }

    void	PressKey(CDC* pDC, int status)
    {
        if (status != pressed)
        {
            pressed = status;
            Draw(pDC);
        }
    }
      
	void	Draw(CDC* pDC)
	{         
		COLORREF color;
		if (lighted) color = downBordCol;
		else color = upBordCol;

		CPen pen(PS_SOLID, 1, color);
		CBrush brush(frameCol);
		CPen* pOldPen = pDC->SelectObject(&pen);
		CBrush* pOldBrush = pDC->SelectObject(&brush); 

		if (lighted)
		{
			pDC->Rectangle(x, y, x + w, y + h);
			pDC->Rectangle(x+1, y+1, x + w-1, y + h-1);
		}
		else 
		{
			pDC->Rectangle(x, y, x + w, y + h);
		}

		if (pressed) color = lightedKeyCol;
		else color = keyCol;
		CPen pen2(PS_SOLID, 2, keyColBorder);
		CBrush brush2(color);
		pDC->SelectObject (pen2);
		pDC->SelectObject (brush2);
	    
		if (pressed)
		{
			pDC->Rectangle(x + b + 2, y + b + 2, x + w - b - 1, y + h - b - 1);
			pDC->Draw3dRect(x + b, y + b, w - b * 2 - 1, h - b * 2 - 1, upCol1, upCol2);
		}
		else
		{
			pDC->Rectangle(x + b + 2, y + b + 2, x + w - b - 2, y + h - b - 2);
			pDC->Draw3dRect(x + b + 1, y + b + 1, w - b * 2 - 1, h - b * 2 - 1, downCol1, downCol2);
			pDC->Draw3dRect(x + b + 1, y + b + 1, w - b * 2, h - b * 2, downCol1, grey3);
		}
	    
		pDC->SelectObject(pOldPen);
		pDC->SelectObject(pOldBrush);
	   
		title.Draw(pDC);
	}
};

/*================================================================================
===== CKeyboard ===============================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CKeyboard : public CObject
{
protected:
    int bl, bu, br, bd; // Borders

    int kx, ky;         // Keys upper left corner
    int kw, kh;         // Keys width and height
    int w, h, b, s;     // Width, height, border and spacing of each key
   
    BOOL vert;          // Vertical or horizzontal keyboard
    int type;           // 0 = bistable / 1 = interlock / 2 = pulse
       
    COLORREF k_frameCol;        //  external rectangle
    COLORREF k_upBordCol;       //  key released external rectangle border
    COLORREF k_downBordCol;     //  key pressed external rectangle border

    COLORREF k_keyCol;          //  key face
    COLORREF k_keyColBorder;    //  key face border
    COLORREF k_lightedKeyCol;   //  lighted key face
    
    COLORREF k_upCol1;          //  key released left-up border 
    COLORREF k_upCol2;          //  key released right-down border
    COLORREF k_downCol1;        //  key pressed left-up border
    COLORREF k_downCol2;        //  key pressed right-down border

	bool k_transparent;			// keys are drawn in transparent mode 
								// ( with system-clear-type smooting some text must be drawn with transp. = false )
	CString k_fontname;
	int k_font_xsize;			// font x size
	int k_font_ysize;			// font y size
	int k_font_weight;			// font weight
	COLORREF k_textcolor;
	COLORREF k_textshadow;

    CText title;

    CArray  <CKey, CKey&> keyArray;

public:
    CFrame  frame;
    int keyPressed;
	int OldKeyPressed;

    CKeyboard() {};
   
    ~CKeyboard() {}

    
    // Keyboard frame upper left position
    // Width and height of each key
    // Keys border and spacing 
    // Vertical or horizzontal keyboard. 
    // Type of keybord 0 = bistable / 1 = interlock / 2 = pulse
    void Init(int _fx, int _fy, int _w, int _h, int _b, int _s, BOOL _vert, int _type)
    {
        frame.x = _fx;
        frame.y = _fy;
        w = _w;
        h = _h;
        b = _b;
        s = _s;
        vert = _vert;
        type = _type;
        kw = 0;
        kh = 0;
    }

    void InitTitle( int tx, int ty, 
				    int font_xsize, int font_ysize, int font_weight,
				    CString fname,
					COLORREF color, COLORREF shadow, bool transparent,
					CString text)
    {
        title.Init(kx + tx, ky + ty, font_xsize, font_ysize, font_weight, fname, color, shadow, transparent);
        title.Set(text);
    }


    // Frame border left / right / up / down and radius of corners
    void InitFrame(int _bl, int _bu, int _br, int _bd, int _fr)                      
    {
        bl = _bl;
        bu = _bu;
        br = _br;
        bd = _bd;
        frame.rad = _fr;
        frame.w = bl + br;
        frame.h = bu + bd;
        kx = frame.x + bl;
        ky = frame.y + bu;
    }

    void InitColors(COLORREF _fBorder, COLORREF _fColor)
    {
        frame.border = _fBorder;
        frame.color = _fColor;
    }

    void InitKeyColors(COLORREF _frameCol,
                       COLORREF _upBordCol, COLORREF _downBordCol,
                       COLORREF _keyCol,
                       COLORREF _keyColBorder,
                       COLORREF _lightedKeyCol,
                       COLORREF _upCol1,   COLORREF _upCol2,
                       COLORREF _downCol1, COLORREF _downCol2)
    {
        k_frameCol = _frameCol;
        k_upBordCol = _upBordCol;
        k_downBordCol = _downBordCol;
        k_keyCol = _keyCol;
        k_keyColBorder = _keyColBorder;
        k_lightedKeyCol = _lightedKeyCol;   
        k_upCol1 = _upCol1;
        k_upCol2 = _upCol2;
        k_downCol1 = _downCol1;
        k_downCol2 = _downCol2;
    }

	void InitKeyFont(CString _fontname,
						bool _transparent,
						int _font_xsize,
						int _font_ysize,
						int _font_weight,
						COLORREF _textcolor,
						COLORREF _textshadow)
	{
			k_fontname = _fontname;
			k_transparent = _transparent;
			k_font_xsize = _font_xsize;
			k_font_ysize = _font_ysize;
			k_font_weight = _font_weight;
			k_textcolor = _textcolor;
			k_textshadow = _textshadow;
	}
    
    void AddKey(CString t, int tx, int ty)
    {
        int nx, ny;
        if (vert)
        {
            nx = kx;
            ny = ky + kh;
            kw = w + b * 2;
            kh += (h + b * 2) + s;
            frame.w = bl + br + kw;
            frame.h = bu + bd + kh - s;
        }
        else 
        {
            nx = kx + kw;
            ny = ky;
            kw += (w + b * 2) + s;
            kh = h + b * 2;   
            frame.w = bl + br + kw - s;
            frame.h = bu + bd + kh;
        }  
        
		CKey ck;
        keyArray.Add(ck);
        int k = keyArray.GetSize() - 1;
        keyArray[k].Init(nx, ny, w, h, b, type);
        keyArray[k].InitText(t, tx, ty, k_font_xsize, k_font_ysize, k_font_weight, k_textcolor, k_textshadow, k_transparent, k_fontname);
        keyArray[k].InitColors(k_frameCol, 
			                   k_upBordCol, k_downBordCol,
                               k_keyCol, k_keyColBorder, k_lightedKeyCol,
                               k_upCol1, k_upCol2, k_downCol1, k_downCol2);
        if (keyPressed == k) keyArray[k].pressed = TRUE;
    }

	void DeleteAllKeys()
    {
		keyArray.RemoveAll();
	}

    BOOL Inside(CPoint point)
    {
        if (point.x < kx || point.x > kx + kw ||
            point.y < ky || point.y > ky + kh) return FALSE;
        return TRUE;
    }                                 

    BOOL ClickInside(CDC* pDC, CPoint point)
    {
        if (point.x < kx || point.x > kx + kw ||
            point.y < ky || point.y > ky + kh) return FALSE;
                 
        int key = PointedKey(point);

        if (type == 1 && key == keyPressed) return FALSE;

        if ( keyArray[key].ClickInside(pDC, point))
        {       
			OldKeyPressed = keyPressed;
            ReleaseOtherKeys(pDC, key);
            // If interlocked release interlocked keys
            if (type == 1)
            {
                for (int i = 0; i < keyArray.GetSize(); i++)
                {
                    if (i != key && keyArray[i].pressed)
                    {
                        keyArray[i].pressed = FALSE;
                        keyArray[i].Draw(pDC);
                    }
                }
            }
            keyPressed = key;
            return TRUE;
        }
        return FALSE;
    }

    void ReleaseOtherKeys(CDC* pDC, int key)
    {
        // If interlocked release other keys
        if (type == 1)
        {
            for (int i = 0; i < keyArray.GetSize(); i++)
            {
                if (i != key && keyArray[i].pressed)
                {
                    keyArray[i].pressed = FALSE;
                    keyArray[i].Draw(pDC);
                }
            }
        }
    }

    int PointedKey(CPoint point)
    {
        int numkeys = keyArray.GetSize();
        int key;
        if (vert) key = (point.y - ky) / (h + b * 2 + s);
        else key = (point.x - kx) / (w + b * 2 + s);    
        if (key < 0) key = 0;
        if (key >= numkeys) key = numkeys - 1;
        return key;
    }

    void ReleaseAllKeys(CDC* pDC)
    {
        for (int i = 0; i < keyArray.GetSize(); i++)
        {
            if (keyArray[i].pressed)
            {
                keyArray[i].pressed = FALSE;
                if (pDC) keyArray[i].Draw(pDC);
            }
        }
        keyPressed = -1;
    }

    void UnlightAllKeys(CDC* pDC)
    {
        for (int i = 0; i < keyArray.GetSize(); i++)
        {
            if (keyArray[i].lighted)
            {
                keyArray[i].lighted = FALSE;
                if (pDC) keyArray[i].Draw(pDC);
            }
        }
    }

    void LightKey(CDC* pDC, int nkey)
    {
        if (nkey >= 0 && nkey < keyArray.GetSize())
        {
            keyArray[nkey].lighted = TRUE;
            if (pDC) keyArray[nkey].Draw(pDC);
        }
    }

    void PressKey(CDC* pDC, int nkey)
    {
        if (nkey == keyPressed) return;
        if (nkey >= 0 && nkey < keyArray.GetSize())
        {
            ReleaseOtherKeys(pDC, nkey);        
            keyArray[nkey].pressed = TRUE;
            keyArray[nkey].Draw(pDC);
            keyPressed = nkey;
        }
        if (nkey < 0) ReleaseAllKeys(pDC);
    }
       
    void Draw(CDC* pDC)
    {
        if (br && bd) frame.Draw(pDC);

        title.Draw(pDC);

        for (int i = 0; i < keyArray.GetSize(); i++)
        {
            keyArray[i].Draw(pDC);
        }
    }

    void Update(CDC* pDC)
    {
        for (int i = 0; i < keyArray.GetSize(); i++)
        {
            keyArray[i].Draw(pDC);
        }
    }
};



/*================================================================================
===== CIndicator ==============================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CIndicator : public CObject 
{
protected:
    int x, y;
	int xFirstLetter;
	UINT blinkcounter;
	bool blinkstatus;
	int xsize;
	int ysize;
	int maxChars;
	int startCursorPosition;

public:
    CIndicator() {}
    ~CIndicator() {}

	BOOL editing;
	CString strValue;
	int cursorPos;
	int selectionStart;
	int selectionEnd;

    void Init(int _x, int _y)
    {
        x = _x;                
        y = _y;
		editing = false;
		cursorPos = -1;
		selectionStart = -1;
		selectionEnd = -1;
		xsize = 114;
		ysize = 26;
		maxChars = 9;
    }

    BOOL Inside(CPoint point)
    {
        if (point.x > x && point.x < x + xsize &&
            point.y > y && point.y < y + ysize) return TRUE;
        return FALSE;
    }

	BOOL ClickInside(CDC* pDC, CPoint point)
    {
        if (point.x < x || point.x > x + xsize ||
            point.y < y || point.y > y + ysize) return FALSE;

		int len = strValue.GetLength();

		if (editing) 
		{
			selectionStart = (point.x - xFirstLetter + 5) / 10;
		}
		else
		{
			CString trimmedString = strValue;
			trimmedString.TrimLeft();
			int initialSpaces = len - trimmedString.GetLength();
			selectionStart = (point.x - x) / 10 - initialSpaces;
			if (trimmedString.Left(2) == "+ " || trimmedString.Left(2) == "- ")
			{
				if (selectionStart > 1) selectionStart--;
			}
		}

		if (selectionStart < 0) selectionStart = 0;
		if (selectionStart > len) selectionStart = len; 
		selectionEnd = selectionStart;
		cursorPos = selectionStart;
		startCursorPosition = cursorPos;
		blinkstatus = true;
		Draw(pDC, 0, 0);

		return TRUE;
	}

	void MouseMove(CDC* pDC, CPoint point)
	{
		cursorPos = (point.x - xFirstLetter + 5) / 10;

		if (cursorPos < 0) cursorPos = 0;
		if (cursorPos > strValue.GetLength()) cursorPos = strValue.GetLength();

		if (cursorPos <= startCursorPosition) selectionStart = cursorPos;
		if (cursorPos >= startCursorPosition) selectionEnd = cursorPos;
	
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void SelectAll(CDC* pDC)
	{
		selectionStart = 0;
		selectionEnd = strValue.GetLength();
		cursorPos = selectionEnd;
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void Update(CDC* pDC)
	{
		if (editing)
		{
			if (blinkcounter++ > 5)
			{
				blinkcounter = 0;
				blinkstatus = !blinkstatus;
				Draw(pDC, 0, 0);
			}
		}
	}

	void AddText(CDC* pDC, CString s)
	{
		// ------------------------------------------------ numeric only
		s.Replace(",", ".");
		int i;
		for (i = 0; i < s.GetLength(); i++)
		{
			if ((s.GetAt(i) < '0' || s.GetAt(i) > '9') && s.GetAt(i) != '.')
				s.SetAt(i, ' ');
		}
		s.Replace(" ", "");
		
		// ------------------------------------------------ limit selection sart and end
		if (selectionStart > strValue.GetLength()) selectionStart = strValue.GetLength();
		if (selectionEnd > strValue.GetLength()) selectionEnd = strValue.GetLength();

		// ------------------------------------------------ insert the string
		strValue = strValue.Left(selectionStart) + s + strValue.Mid(selectionEnd);

		// ------------------------------------------------ limit to nchars
		strValue = strValue.Left(maxChars);

		selectionStart += s.GetLength();
		if (selectionStart > maxChars) selectionStart = maxChars;

		selectionEnd = selectionStart;
		cursorPos = selectionStart;

		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	CString GetSelectedText()
	{
		return strValue.Mid(selectionStart, selectionEnd - selectionStart);
	}

	void ClearSelectedText(CDC* pDC)
	{
		strValue = strValue.Left(selectionStart) + strValue.Mid(selectionEnd);
		selectionEnd = selectionStart;
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void KeyLeft(CDC* pDC)
	{
		if (cursorPos > 0) 
		{
			cursorPos--;
			if (KeyPressed(VK_CONTROL)) cursorPos = 0;		
			if (KeyPressed(VK_SHIFT)) 
			{
				if (cursorPos < selectionStart) selectionStart = cursorPos;
				else selectionEnd = cursorPos;
			}
			else
			{
				selectionEnd = selectionStart = cursorPos;
			}
		}
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void KeyRight(CDC* pDC)
	{
		if (cursorPos < strValue.GetLength()) 
		{
			cursorPos++;
			if (KeyPressed(VK_CONTROL)) cursorPos = strValue.GetLength();
			if (KeyPressed(VK_SHIFT))
			{
				if (cursorPos > selectionEnd) selectionEnd = cursorPos;
				else  selectionStart = cursorPos;
			}
			else
			{
				selectionStart = selectionEnd = cursorPos;
			}
		}
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void Backspace(CDC* pDC)
	{
		if (selectionEnd == selectionStart) selectionStart--;
		if (selectionStart < 0) selectionStart = 0;
		if (selectionEnd > strValue.GetLength()) selectionEnd = strValue.GetLength();
		strValue = strValue.Left(selectionStart) + strValue.Mid(selectionEnd); 
		cursorPos = selectionStart;
		selectionEnd = selectionStart;
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void Delete(CDC* pDC)
	{
		if (selectionEnd == selectionStart) selectionEnd++;
		if (selectionStart < 0) selectionStart = 0;
		if (selectionEnd > strValue.GetLength()) selectionEnd = strValue.GetLength();
		strValue = strValue.Left(selectionStart) + strValue.Mid(selectionEnd);
		cursorPos = selectionStart;
		selectionEnd = selectionStart;
		blinkstatus = true;
		Draw(pDC, 0, 0);
	}

	void Draw(CDC* pDC, double v, int type)
	{
		if (type < 0) return;

		CString str;

		if (!editing) 
		{
			//double kv = v / 1000;
			switch (type)
			{
				case REGTYPE_FREQ:
				case REGTYPE_FMAX:
				case REGTYPE_FMIN:
					if (v >= 100000) str.Format("%1.0f Hz", v);
					else if (v >= 10000) str.Format("%1.1f Hz", v);
					else if (v >= 1000) str.Format(" %1.1f Hz", v);
					else if (v >= 100) str.Format("  %1.1f Hz", v);
					else if (v >= 10) str.Format("  %1.2f Hz", v);
					else str.Format("  %1.3f Hz", v);
					break;
				case REGTYPE_SLOT:
					if (v > 0) str.Format("First =%3.0f", v);
					else str = " Disabled";
					break;
				case REGTYPE_CALIB:
					if (v >= 10) str.Format("%2.3f V", v);
					else str.Format("%2.3f volt", v); 
					break;
		
				case REGTYPE_DUTY:
					str.Format("  %4.1f %%", v);
					break;
				case REGTYPE_BURST:
				case REGTYPE_PAUSE:
					str.Format("%3.0f cycles", v);
					break;
				case REGTYPE_BEGIN:
					str.Format(" %5.2f mS", v);
					break;
				case REGTYPE_END:
					str.Format(" %5.1f mS", v);
					break;
				case REGTYPE_TIME:
					if (v >= 100) str.Format("%5.0f  sec", v);
					else if (v >= 10) str.Format("%5.1f  sec", v);
					else str.Format(" %5.2f sec", v);
					break;
					
				case REGTYPE_DBMAX:  
				case REGTYPE_DBMIN:
				case REGTYPE_OUTLEV:
					if (v >= 10) str.Format(" +%3.1f dB", v);
					else if (v >= 0) str.Format(" + %3.1f dB", v);
					else if (v > -10) str.Format(" - %3.1f dB", -v);
					else str.Format(" -%3.1f dB", -v);
					break;
			}
			strValue = str;
		}

		//pDC->SetBkMode (OPAQUE);
		pDC->SetBkMode (TRANSPARENT);
		pDC->Draw3dRect(x - 1, y - 1, xsize - 2, ysize - 2, 0, grey8);
		pDC->Draw3dRect(x - 2, y - 2, xsize, ysize, 0, grey8);

		if (editing) 
		{
			// ------------------------------------------------------ remove background
			pDC->FillSolidRect(x, y, xsize - 4, ysize - 4, RGB(0,90,0));	
			pDC->SetBkColor(RGB(0,70,120));
			pDC->SetTextColor(RGB(255, 255, 50));
			// ------------------------------------------------------ center the string
			xFirstLetter = x + (xsize - 4 - strValue.GetLength() * 10) / 2;
			// ------------------------------------------------------ limit cursor and selections to the string
			if (cursorPos > strValue.GetLength())
			{
				cursorPos = strValue.GetLength();
				selectionEnd = cursorPos;
				selectionStart = cursorPos;
			}
			// ------------------------------------------------------ show selection in bue color
			if (selectionEnd > selectionStart) 
			{
				int x1 = xFirstLetter + selectionStart * 10 - 1;
				int dx = (selectionEnd - selectionStart) * 10 + 1;
				pDC->FillSolidRect(x1, y, dx, 22, RGB(50,120,255));
			}
			// ------------------------------------------------------ print the string
			CFont font;
			font.CreateFont(24, 10, 0, 0, 1800, 0, 0, 0, 0, 0, 0, CLEARTYPE_QUALITY, 0, "Courier new");
			CFont* pOldFont = pDC->SelectObject (&font);
			pDC->TextOut(xFirstLetter, y, strValue);
			pDC->SelectObject (pOldFont);
			// ------------------------------------------------------ blink the cursor
			if (blinkstatus)
			{
				int x1 = xFirstLetter + cursorPos * 10;
				if (x1 > x + xsize - 2) x1 = x + xsize - 2;
				pDC->FillSolidRect(x1, y, 2, 20, RGB(255,255,255));
			}
		}
		else
		{
			pDC->FillSolidRect(x, y, 110, 22, RGB(0,70,0));
			pDC->SetBkColor(RGB(0,70,0));
			pDC->SetTextColor(RGB(90, 240, 85));
			CFont font;
			font.CreateFont(24, 10, 0, 0, 1800, 0, 0, 0, 0, 0, 0, CLEARTYPE_QUALITY,  0, "Courier new");
			CFont* pOldFont = pDC->SelectObject (&font);
			pDC->TextOut(x+4, y, str);
			pDC->SelectObject (pOldFont);
		}
	}
};


/*================================================================================
===== CDialer =================================|  CLASS                           |
                                                ==================================
   Description    .                     
=================================================================================*/

class CDialer : public CObject 
{
protected:
    int     x, y, r, r2, r3, xa, ya;
    int     step, nstep;
    double  v, val, amin, amax, vmin, vmax;    
    double  power, multiplier, precision;
    double* pVal;
    
    int     rS, r2S, r3S, xaS, yaS;
    int     stepS, nstepS;
    double  vS, valS, aminS, amaxS, vminS, vmaxS;    
    double  powerS, multiplierS, precisionS;
    double* pValS;

	BOOL    notZeroWarning;
    BOOL    stopcursor;
    CArray  <CText, CText&> textArray;
    CArray  <double, double&> valuesArray;
    
    double  damax;

    BOOL    dual;
    int     knob;

    BOOL dcCreated;
    CDC dc1, dc2;
    CBitmap bmp1, bmp2;

    CText title;
	COLORREF titleColor;
	COLORREF titleShadow;

	double mouseWheelStep;
	BOOL mouseWheelSpecial;

	BOOL filterKnob;


public:
    double a, aS;
	BOOL allowMicrometricMouse;

    CDialer()
    {
        dcCreated = FALSE;
        dual = FALSE;
        knob = 0;
        a = 0;
        pVal = 0;
        amin = -1;
        amax = -1;
        aS = 0;
        pValS = 0;
        aminS = -1;
        amaxS = -1;
		notZeroWarning = FALSE;
		mouseWheelStep = 0;
		mouseWheelSpecial = FALSE;
		allowMicrometricMouse = FALSE;
		filterKnob = FALSE;
    }

    ~CDialer() {}         

    void Init(int _x, int _y, int _r, int _r2, int _r3)
    {
        x = _x;
        y = _y;
        r = _r;
        r2 = _r2;
        r3 = _r3;
    }

    void InitDual(int _r, int _r2, int _r3)
    {
        rS = _r;
        r2S = _r2;
        r3S = _r3;
        dual = TRUE;
    }   

    void SetValue(double _v)
    {
		if (_v > vmax) _v = vmax;
		if (_v < vmin) _v = vmin;
        val = v = _v;
        *pVal = v;
    }   

    void SetValueS(double _v)
    {
		if (_v > vmaxS) _v = vmaxS;
		if (_v < vminS) _v = vminS;
		valS = vS = _v;
		*pValS = vS;
	}

	void ChangeValue(double deltaV)
    {
		if (dual)
		{
			if (abs(deltaV) >= 1)
			{
				step += int(deltaV);
				if (step < 0) step = 0;
				if (step >= nstep - 1) step = nstep - 1;
				SetValue(valuesArray[step]); 
			}
			else
			{
				valS += deltaV * (vmaxS - vminS);
				if (valS > vmaxS) valS = vmaxS;
				if (valS < vminS) valS = vminS;
				*pValS = valS;
			}
		}
		else
		{
			// ---------------------------------------------------------------- mouse wheel special
			if (mouseWheelSpecial) deltaV *= multiplier;
			// ---------------------------------------------------------------- mouse wheel normal
			if (mouseWheelStep != 0) SetValue(v + deltaV * mouseWheelStep);
			// ---------------------------------------------------------------- mouse wheel step not defined
			else SetValue(v + deltaV * (vmax - vmin) / 10.0);
		}
    }   

    void UpdatePosition(CDC* pDC) 
    {
        val = v = *pVal;
        if (amin >= 0 || amax >=0)
        {
            if (valuesArray.GetSize())
            {
                for (int i = 0; i < nstep; i++)
                { 
                    if (val == valuesArray[i]) step = i;
                }
                a = amin + step * (amax - amin) / (nstep - 1);
            }
            else
            {
                a = (val - vmin) * (amax - amin) / (vmax - vmin) + amin;
            }
        }
		else
		{
			a = val / abs(vmax - vmin) * 360.0;
		}

        if (dual)
        {
            valS = vS = *pValS;
            if (aminS >= 0 || amaxS >=0)
            {        
                aS = (valS - vminS) * (amaxS - aminS) / (vmaxS - vminS) + aminS;  
            }
        }
        //if (amin >= 0 || amax >=0 || aminS >= 0 || amaxS >=0) Update(pDC);
        Update(pDC);
    }   

    void SetMultiplier(double _multiplier)
    {
        multiplier = _multiplier;
    }

	void SetMouseWheelParams(double _mouseWheelStep, BOOL _mouseWheelSpecial)
    {
        mouseWheelStep = _mouseWheelStep;
		mouseWheelSpecial = _mouseWheelSpecial;
    }
 
    void InitRange(double _vmin, double _vmax,
                   double _power, double _multiplier, double _precision,
                   double* _pVal)
    {
        vmin = _vmin;
        vmax = _vmax;
        power = _power;
        multiplier = _multiplier;
        precision = _precision;
        pVal = _pVal;
        v = *pVal;
        val = v;
    }

    void InitRangeDual(double _vmin, double _vmax,
                        double _power, double _multiplier, double _precision,
                        double* _pVal)
    {
        vminS = _vmin;
        vmaxS = _vmax;
        powerS = _power;
        multiplierS = _multiplier;
        precisionS = _precision;
        pValS = _pVal;
        vS = *pValS;
        valS = vS;
    }


    // To have a multi-turn dialer do not call InitScale
    // (InitScale must be called after InitRange)
    void InitScale(double _amin, double _amax, int _nstep)
    {
        amin = _amin;
        amax = _amax;
        nstep = _nstep;
        a = (val - vmin) * (amax - amin) / (vmax - vmin) + amin;
    }
    void InitScaleDual(double _amin, double _amax, int _nstep)
    {
        aminS = _amin;
        amaxS = _amax;
        nstepS = _nstep;
        aS = (valS - vminS) * (amaxS - aminS) / (vmaxS - vminS) + aminS;
    }


    // InitScaleText adds nstep labels around the dialer
    // Labels must be separated with commas in the text string
    // (InitScaleText must be called after InitScale)
	void InitScaleText(int dx, int dy, int dr, 
					   int font_xsize, int font_ysize, int font_weight,
					   CString fname, COLORREF color, COLORREF shadow, bool transparent, CString text)
	{
		textArray.RemoveAll();
		double ang = amin;
		double da = (amax - amin) / (nstep - 1);
		CText t;
		int ix;
		for (int i = 0; i < nstep; i++)
		{ 
			double rad = (ang + 90) * PI / 180;
			int xa = int(x + dx + (r + dr) * cos(rad) + .5);
			int ya = int(y + dy + (r + dr) * sin(rad) + .5);
			ang += da;
			ix = text.Find(",");
			if (ix < 0) ix = text.GetLength();
			t.Init(xa, ya, font_xsize, font_ysize, font_weight, fname, color, shadow, transparent);
			t.Set(text.Left(ix));
			textArray.Add(t);
	           
			if (text.GetLength() > ix) text = text.Mid(ix + 1);
		}
	}    


	// InitScaleValues sets a numeric value to each label
    // and the knob can assume only nstep positions
    // (InitScaleValues must be called after InitScaleText)
	void InitScaleValues(CString values)
	{
		step = 0;
		valuesArray.RemoveAll();
		int ix;
		double value;
		for (int i = 0; i < nstep; i++)
		{ 
			ix = values.Find(",");
			if (ix < 0) ix = values.GetLength();
			value = atof(values.Left(ix));
			if (value == val) step = i;
			valuesArray.Add(value);
			if (values.GetLength() > ix)   values = values.Mid(ix + 1);
		}
		a = amin + step * (amax - amin) / (nstep - 1);
	}


    void InitTitle(int tx, int ty,
				   int font_xsize, int font_ysize, int font_weight,
                   CString fname,
                   COLORREF color, COLORREF shadow, bool transparent,
                   CString text)
    {
		titleColor = color;
		titleShadow = shadow;
        title.Init(x + tx, y + ty, font_xsize, font_ysize, font_weight, fname, color, shadow, transparent);
        title.Set(text);
    }

	void SetNotZeroWarning()
    {
        notZeroWarning = TRUE;
	}

	void SetFilterKnob(BOOL _filterKnob)
    {
        filterKnob = _filterKnob;
	}
   
	inline int Inside(CPoint point)  
	{
		if (point.x < x - r || point.x > x + r ||
			point.y < y - r || point.y > y + r) return 0;   
		if (dual)
		{
			int dx = abs(point.x - x);
			int dy = abs(point.y - y);
			double dist = sqrt(double(dx * dx + dy * dy));
			if (dist <= rS) return 2;
		}
		return 1;
	}

	void	Draw(CDC* pDC)   
	{
		if (!dcCreated)
		{
			bmp1.CreateCompatibleBitmap(pDC, r*2, r*2);
			dc1.CreateCompatibleDC(pDC);
			dc1.SelectObject(bmp1);
			bmp2.CreateCompatibleBitmap(pDC, r*2, r*2);
			dc2.CreateCompatibleDC(pDC);
			dc2.SelectObject(bmp2);
			dcCreated = TRUE;
		}

		AntialiasedCircle(pDC, x + 4, y + 4, r-2, 1, 4, RGB(180,180,180));		// External shadow
		AntialiasedCircle(pDC, x + 2, y + 2, r-3, 1, 5, RGB(90,90,90));			// Internal shadow
		AntialiasedCircle(pDC, x, y, r-2, .5, 1, black);						// Black border

		//if (dual)
		//{
		//	// ------------------------------------------------------ CALIBRATE INDICATOR
		//	int rr = int(r * 1.62);
		//	int xf = x + int(rr * sin(310 * PI / 180));
		//	int yf = y + int(rr * cos(310 * PI / 180));
		//	int xs = x + int(rr * sin(325 * PI / 180));
		//	int ys = y + int(rr * cos(325 * PI / 180));
		//	CPen pen(PS_SOLID, 1, RGB(75,75,75));
		//	CPen* pOldPen = pDC->SelectObject(&pen);
		//	pDC->Arc(x - rr, y - rr, x + rr, y + rr, xf, yf, xs, ys);

		//	pDC->MoveTo(xs-15, ys-12);
		//	pDC->LineTo(xs, ys);

		//	pDC->MoveTo(xs, ys);
		//	pDC->LineTo(xs , ys - 5);
		//	pDC->MoveTo(xs, ys);
		//	pDC->LineTo(xs - 5, ys);
		//	CFont font;
		//	font.CreateFont(10, 0, 0, 0, 1500, 0, 0, 0, 0, 0, 0, CLEARTYPE_QUALITY, 0, "Arial");
		//	CFont* pOldFont = pDC->SelectObject(&font);
		//	if (filterKnob)
		//	{
		//		pDC->SetTextColor(RGB(180, 40, 0));
		//		pDC->TextOut(xs - 34, ys, "FILTER.");
		//	}
		//	else
		//	{
		//		pDC->SetTextColor(RGB(80, 80, 80));
		//		pDC->TextOut(xs - 20, ys, "CAL.");
		//	}
		//	pDC->SelectObject(pOldFont);
		//	pDC->SelectObject(pOldPen);
		//}
	    

		for (int i = 0; i < textArray.GetSize(); i++)
		{
			textArray[i].Draw(pDC);
		}

		if (dcCreated) dc1.BitBlt(0, 0, r*2, r*2, pDC, x - r, y - r, SRCCOPY);


		AntialiasedCircleFilled(&dc1, r, r, r2 + r3 + 1, 1, grey6);				// Face color

		if (dual)
		{
			AntialiasedCircle(&dc1, r+3, r+2, rS-2, 1, 3, RGB(110,110,110));	// Shadow
			AntialiasedCircle(&dc1, r, r, rS - 2, 1, 2, black);					// Black border
			AntialiasedCircleFilled(&dc1, r, r, r2S + r3S + 2, 2, grey6);		// Face color
		}

		title.Draw(pDC);
		Update(pDC);
	}
	                             

	void	Update(CDC* pDC)  
	{     
		dc2.BitBlt(0, 0, r*2, r*2, &dc1, 0, 0, SRCCOPY);

		double rad = (a + 90) * PI / 180;
		xa = int(floor(x + r2 * cos(rad) + .5));
		ya = int(floor(y + r2 * sin(rad) + .5));   
		int xa2 = xa - x + r;
		int ya2 = ya - y + r;
		AntialiasedCircleFilled(&dc2, xa2, ya2, r3, 2, black);
		AntialiasedCircleFilled(&dc2, xa2, ya2, r3-1.5, 2, grey5);
	    
		int xa3 = 0;
		int ya3 = 0;
		if (dual)
		{
			rad = (aS + 90) * PI / 180;
			xaS = int(floor(x + r2S * cos(rad) + .5));
			yaS = int(floor(y + r2S * sin(rad) + .5));   
			xa3 = xaS - x + r;
			ya3 = yaS - y + r;
			AntialiasedCircleFilled(&dc2, xa3, ya3, r3S, 2, black);
			AntialiasedCircleFilled(&dc2, xa3, ya3, r3S-1.5, 2, grey5);
		}

		AntialiasedCircleFilled(&dc2, xa2- r3/4, ya2- r3/4, r3/4, 1, grey6);
		if (dual) AntialiasedCircleFilled(&dc2, xa3-r3S/4, ya3-r3S/4, r3S/4, 1, grey6);

		if (notZeroWarning)
		{
			if (abs(v) >= precision)
			{
				title.InitColors(RGB(160, 50, 0), RGB(255, 200, 160), TRUE);
			}
			else
			{
				title.InitColors(titleColor, titleShadow, FALSE);
			}
			title.Draw(pDC);
		}

		if (dual)
		{
			// ------------------------------------------------------ CALIBRATE INDICATOR
			
			// ------------------------------------------ arrow
			COLORREF c1 = RGB(75,75,75);
			if (filterKnob) c1 = RGB(160,50,0);
			CPen pen(PS_SOLID, 1, c1);
			CPen* pOldPen = pDC->SelectObject(&pen);
			// ------------------------------------------ arrow start and final points
			int rr = int(r * 1.67);
			int xf = x + int(rr * sin(310 * PI / 180));
			int yf = y + int(rr * cos(310 * PI / 180));
			int xs = x + int(rr * sin(325 * PI / 180));
			int ys = y + int(rr * cos(325 * PI / 180));
			xf -= 2;
			xs -= 2;
			// ------------------------------------------ arrow bar
			pDC->MoveTo(xf, yf - 1);
			pDC->LineTo(xs, ys);
			// ------------------------------------------ arrow point
			pDC->MoveTo(xs, ys);
			pDC->LineTo(xs, ys - 5);
			pDC->MoveTo(xs, ys);
			pDC->LineTo(xs - 5, ys);

			CFont font;
			font.CreateFont(10, 0, 0, 0, 1500, 0, 0, 0, 0, 0, 0, CLEARTYPE_QUALITY, 0, "Arial");
			CFont* pOldFont = pDC->SelectObject(&font);
			pDC->SetBkMode(OPAQUE);
			pDC->SetBkColor(panels);

			// ------------------------------------------- text
			if (filterKnob)
			{
				pDC->SetTextColor(RGB(180, 40, 0));
				pDC->TextOut(xs - 31, ys + 1, "FILTER.");
			}
			else
			{
				pDC->SetTextColor(RGB(80, 80, 80));
				pDC->TextOut(xs - 31, ys + 1, "      CAL.");
			}

			pDC->SelectObject(pOldFont);
			pDC->SelectObject(pOldPen);
		}

		pDC->BitBlt(x - r, y - r, r*2, r*2, &dc2, 0, 0, SRCCOPY);
	}


	BOOL ClickInside(CDC* pDC, CPoint point)  
	{
		int dx = abs(point.x - x);
		int dy = abs(point.y - y);
		double dist = sqrt(double(dx * dx + dy * dy));
		if (dist > r) return FALSE;
		if (dist > rS) knob = 0;
		else knob = 1;

		RECT rect;
		CWnd* pWnd = pDC->GetWindow();
		pWnd->GetClientRect(&rect);
		pWnd->ClientToScreen(&rect);

		// Removed in version 4 to get working the DoubleClick for dialers
		//if (knob == 0) SetCursorPos(rect.left + xa, rect.top + ya);
		//else SetCursorPos(rect.left + xaS, rect.top + yaS);
	    
		damax = 0;

		return TRUE;
	}

	double DxDyToAngle (int dx, int dy)
	{
	   double a;
	   if (abs(dy) < abs(dx))
	   {
		  a = 180.0 * (atan((double)dy / dx) / PI);
		  if (dx < 0) a += 180;
		  if (a < 0) a += 360;
	   }
	   else
	   {
		  a = 90 - 180.0 * (atan((double)dx / dy) / PI);
		  if (dy < 0) a += 180;
	   }
	   return a;
	}

	double	CalcAngle(CPoint point)  
	{
		double angle = DxDyToAngle (point.x - x, point.y - y) - 90; 
		if (angle < 0) angle += 360;
		double da;
	    
		stopcursor = FALSE;
		if (knob == 0)
		{
			da = angle - a;
			if (da < -180) da += 360;
			if (da > 180) da -= 360;

			if (amin >= 0 || amax >=0)          // One-turn dialer
			{
				if (fabs(da) > 180)
				{
					angle = a;
					da = 0;
					stopcursor = TRUE;
				}
				else if (angle < amin)
				{
					angle = amin;
					da = 0;
					stopcursor = TRUE;
				}
				else if (angle > amax)
				{
					angle = amax;
					da = 0;
					stopcursor = TRUE;
				}
			}
			a = angle;
		}
		else
		{
			da = angle - aS;
			if (aminS >= 0 || amaxS >=0)          // One-turn dialer
			{
				if (fabs(da) > 180)
				{
					angle = aS;
					da = 0;
					stopcursor = TRUE;
				}
				else if (angle < aminS)
				{
					angle = aminS;
					da = 0;
					stopcursor = TRUE;
				}
				else if (angle > amaxS)
				{
					angle = amaxS;
					da = 0;
					stopcursor = TRUE;
				}
			}
			aS = angle;
		}

		if (da > 180) da -= 360;
		if (da < -180) da += 360;
	    
		if (knob == 0 && valuesArray.GetSize())   // Locked on labels
		{
			step = int(nstep * (angle - amin) / (amax - amin));
			if (step < 0) step = 0;
			if (step >= nstep) step = nstep - 1;
			a = amin + step * (amax - amin) / (nstep - 1);
			da = 0;
		}
	         
		// DAMPING
		if (fabs(da) > fabs(damax))
		{
			damax += (da - damax) / 10;
			da = damax;
		}
		else
		{
			damax = da;
		}

		return da;
	}

	void	Rotation(CDC* pDC, CPoint point)   
	{
		double da = CalcAngle(point);
		da *= 4;  // knobSpeed;
			
		Update(pDC);
	      
		RECT rect;
		CWnd* pWnd = pDC->GetWindow();
		pWnd->GetClientRect(&rect);
		pWnd->ClientToScreen(&rect);

		//ClipCursor(&rect);

		int dx = abs(point.x - x);
		int dy = abs(point.y - y);
		double dist = sqrt(double(dx * dx + dy * dy));

		// In the external dials (with discrete positions) the cursor is free to move in the area from "r" to "r/2"
		// In the internal, or single dials (with contiunuous positions) the cursor is locked to the knob
		if (dual)
		{
			if (knob == 0)
			{
				if (dist > r || dist < r/2 || stopcursor) SetCursorPos(rect.left + xa, rect.top + ya);
			}
			else
			{
				SetCursorPos(rect.left + xaS, rect.top + yaS);
			}
		}
		else
		{
			SetCursorPos(rect.left + xa, rect.top + ya);
		}

		if (knob == 0 && valuesArray.GetSize()) // Locked on labels
		{
			val = valuesArray[step];
			*pVal = val;
			return;
		}

		if (knob == 0)
		{
			if (amin >= 0)   // One-turn dialer
			{
				v = vmin + (a - amin) * (vmax - vmin) / (amax - amin);
				if (fabs(a - amin) <= 1) v = vmin;
				if (fabs(a - amax) <= 1) v = vmax;
			}
			else             // Multi-turn dialer
			{
				if (da > 0) v += multiplier * pow(da, power);
				else v -= multiplier * pow(-da, power);
			}

			if (v > vmax) v = vmax;
			if (v < vmin) v = vmin;  
			val = floor(v / precision + .5) * precision;

			*pVal = val;   
		}
		else
		{
			if (aminS >= 0)   // One-turn dialer
			{
				vS = vminS + (aS - aminS) * (vmaxS - vminS) / (amaxS - aminS);
				if (fabs(aS - aminS) <= 1) vS = vminS;
				if (fabs(aS - amaxS) <= 1) vS = vmaxS;
			}
			else             // Multi-turn dialer
			{
				if (da > 0) vS += multiplierS * pow(da, powerS);
				else vS -= multiplierS * pow(-da, powerS);
			}

			if (vS > vmaxS) vS = vmaxS;
			if (vS < vminS) vS = vminS;  
			valS = floor(vS / precisionS + .5) * precisionS;

			*pValS = valS;   
		}   
	}

};



// ================================================================================
//  DAA FUNCTIONS                 
// ================================================================================

void DAA_Init();
void DAA_End();

void DAA_InitDefaultValues();

CString DAA_GetDataPath();
void DAA_ReadMainProps();
void DAA_WriteMainProps();
void DAA_ReadAllProps();
void DAA_WriteAllProps();

void DAA_ResetSweep(BOOL resetAll);
void DAA_FillBuffer();

void DAA_InitObjects();

void DAA_FillWaveTable(int type);
void DAA_AddBuffer();

void DAA_CreateInstrument(CDC* pDC);
void DAA_RestorePropsFromLastRun();
void DAA_Draw(CDC* pDC);

BOOL DAA_InputOn();  
void DAA_InputOff();
BOOL DAA_OutputOn();
void DAA_OutputOff();

void DAA_ToggleMultiDraw();
void DAA_LButtonDown(CDC* pDC, CPoint point);
void DAA_LButtonDoubleClick(CDC* pDC, CPoint point);
void DAA_SetLightedKeys(CDC* pDC);
void DAA_SetRegulationType(CDC* pDC, int type);
void DAA_SetTrigType(int type);
void DAA_DrawIndicator(CDC* pDC, int type);
double DAA_GetIndicatorValue(int type);

void DAA_Resize(CDC* pDC, UINT nType, int cx, int cy);
void DAA_LButtonUp(CDC* pDC, CPoint point);
void DAA_MouseMove(CDC* pDC, CPoint point);
void DAA_MouseWheel(CDC* pDC, short zDelta, CPoint point);
void DAA_KeyDown(CDC* pDC, UINT nChar);
void DAA_CTRL_XCV_Keys(UINT nChar);

void DAA_OnIdle(CDC* pDC);
void DAA_Save(CArchive& ar);
void DAA_Load(CArchive& ar);
void DAA_InitAfterLoad();
void DAA_DrawUpdatedDialers(CDC* pDC);
void DAA_PressKeys(CDC* pDC);

void DAA_SetReference();
BOOL DAA_GetReference();
BOOL DAA_GetSpectrum3D();
void DAA_ToggleSpectrum3D();
BOOL DAA_GetMultiDraw();
BOOL DAA_GetDisplaySamples();
void DAA_SetBandwidth(int bw);
int  DAA_GetBandwidth();
void DAA_SetFhtWindow(int fhtwindow);
int  DAA_GetFhtWindow();
void DAA_SetCh1Filter(int n);
int  DAA_GetCh1Filter();
void DAA_SetCh2Filter(int n);
int  DAA_GetCh2Filter();
void DAA_SetFiltersPoles(int n);
int  DAA_GetFiltersPoles();

CBitmap* DAA_GetDisplayBitmap();

void DAA_RegisterDaaAutoFile();
void DAA_ToggleRememberAllProps();
int DAA_GetRememberAllProps();

void DAA_SetInputDevice(int n);
void DAA_SetOutputDevice(int n);

void ClearWindowWithGrayColor(CDC* pDC);
CDialer* GetDialerControl(CPoint point);
void SetDialersAccessoryOperations(CDC* pDC, int dialerIndex);

#endif

