
#include "UtilityFunctions.h"

////////////////////////////////////////////////////////////////
//  FHT INIT
////////////////////////////////////////////////////////////////

void FHTinit(int nn, int w)
{   
    int index, j, s, k;
    double ang = 0;
    double omega = 2 * PI / nn;
    int lg2 = int(log(double(nn)) / log(2.0) + .5);

    double knn = 1.0 / nn; 
    
    for (int i = 0; i < nn; i++) 
    {
		switch(w)
		{
			case 0:
				hTab[i] = float(knn);										// RECTANGULAR
				break;
			case 1:
				hTab[i] = float(sin(ang / 2) * knn * 1.58);					// SIN
				break;
			case 2:
				hTab[i] = float(pow(sin(ang / 2), 2) * knn * 2.01);			// SIN 2   
				break;
			case 3:
				hTab[i] = float(pow(sin(ang / 2), 10) * knn * 4.07);		// SIN 10
				break;
			case 4:
				hTab[i] = float(pow(sin(ang / 2), 100) * knn * 12.6);		// SIN 100
				break;
			case 5:
				hTab[i] = float((1 - cos(ang)) * knn * 0.995);				// HANNING
				break;
			case 6:
                hTab[i] = float((0.62 - 
                                 0.48 * abs(nn / (nn - 1) - 0.5) - 
                                 0.38 * cos(ang)) * knn * 2.62);			// BARTLETT HANN
				break;
			case 7:
				hTab[i] = float((.53836 - 
					             .46164 * cos(ang)) * knn * 1.86);			// HAMMING
				break;
			case 8:
				hTab[i] = float((.42659 - 
					             .49656 * cos(ang) +
								 .076849 * cos(ang * 2)) * knn * 2.34);		// BLACKMAN
				break;
			case 9:
				hTab[i] = float((.40217 - 
								 .49703 * cos(ang) + 
								 .09892 * cos(ang * 2) - 
								 .00188 * cos(ang * 3)) * knn * 2.48);		// NUTTALL NARROW
				break;
			case 10:
				hTab[i] = float((.355768 - 
								 .487396 * cos(ang) + 
								 .144232 * cos(ang * 2) - 
								 .012604 * cos(ang * 3)) * knn * 2.82);		// NUTTALL
				break;
			case 11:
				hTab[i] = float((0.3635819 - 
								 0.4891775 * cos(ang) + 
								 0.1365995 * cos(ang * 2) - 
								 0.0106411 * cos(ang * 3)) * knn * 2.76);	// BlackmanNuttall
				break;
			case 12:
				hTab[i] = float((0.35875 - 
								 0.48829 * cos(ang) + 
								 0.14128 * cos(ang * 2) - 
								 0.01168 * cos(ang * 3)) * knn * 2.79);		// BlackmanHarris
				break;
			case 13:
				hTab[i] = float((0.215578948 -
								 0.41663158 * cos(ang) + 
								 0.277263158 * cos(ang * 2) - 
								 0.083578947 * cos(ang * 3) + 
								 0.006947368 * cos(ang * 4)) * knn * 4.62);	// FlatTop
				break;
		}

        // SIN & COS
        senTab[i] = float(sin(ang));
        cosTab[i] = float(cos(ang));
        ang += omega;

        // PERMUTATION TABLE
        index = i;
        j = 0;
        for (k = 0; k < lg2; k ++)
        {
            s = index >> 1;
            j = j + j + index - s - s; 
            index = s;
        }
        permTab[i] = j;  
    }
  
    initSamples = nn;
}


// ==========================================================================================
// ==========================================================================================
//   FHT 
// ==========================================================================================
// ==========================================================================================

void FHT(int nn, int w)
{
    if (nn > samplesMax) return;

    if (nn != initSamples) FHTinit(nn, w);

    int i, j, le, le2, k, mk, trig_ind, trig_inc;
    float ua, ub, v;
    float* A = aTab;
    float* B = bTab;
    float* C;

	// ======================================================================================
    //	FAST SWEEP & PULSE Samples from circular buffer to aTab
	// ======================================================================================
    if ((analysisType == ANTYPE_FASTSW) || (analysisType == ANTYPE_PULSE))
    {
		// ---------------------------------------------------------- init knn
        double knn = 2.0 / nn;
		if (analysisType == ANTYPE_FASTSW) knn *= 2.38;  // FASTSWEEP level correction
		if (analysisType == ANTYPE_PULSE)  knn *= 1.58;	 // PULSE  level correction

		// ---------------------------------------------------------- init "start" and "nsamp" (BEGIN and END)
        int start = int(SAMPLES_PER_SEC * start_sampling_ms / 1000);
		int nsamp = int(SAMPLES_PER_SEC * sampling_ms / 1000);
		if (nsamp > nn) nsamp = nn;

		// ---------------------------------------------------------- sampling_ms is not used for ANTYPE_FASTSW
		if (analysisType == ANTYPE_FASTSW) nsamp = nn;

		// The channel 1 or 2 is preselected in pulseCbuffPos from the PlotWave function (see: PULSE CONTROL)
		readpos = pulseCbuffPos;
		while (readpos < 0) readpos += IN_CBUF_LEN;

		// ---------------------------------------------------------- move the start sampling point by "start"
        j = readpos + start * numCH;               

		// ---------------------------------------------------------- copy the samples from the circular buffer
        for (i = 0; i < nsamp; i++)
        {
            aTab[permTab[i]] = pCbufIn[j] * float(knn); 
            if ((j += numCH) >= IN_CBUF_LEN) j -= IN_CBUF_LEN;        
        }


		// ============================================================== ZERO CORRECTION ( START to END )
        float corrStart = float(0);
		float corrEnd = float(0);
        // ----------------------------------------- test 1000 samples ( 11mS ) before the trigger position
		j = readpos + start - 1000 * numCH;
		while (readpos < 0) readpos += IN_CBUF_LEN;
		//
        for (i = 0; i < 1000; i++)
        {
            corrStart += pCbufIn[j] * float(knn); 
            if ((j += numCH) >= IN_CBUF_LEN) j -= IN_CBUF_LEN;        
        }
        corrStart = corrStart / float(1000);
		// ------------------------------------------ test 1000 samples ( 11mS ) around the end position					
		j = readpos + start + nsamp - 500 * numCH;
		while (readpos < 0) readpos += IN_CBUF_LEN;
		//
        for (i = 0; i < 1000; i++)
        {
            corrEnd += pCbufIn[j] * float(knn); 
            if ((j += numCH) >= IN_CBUF_LEN) j -= IN_CBUF_LEN;        
        }
        corrEnd = corrEnd / float(1000);
        // ------------------------------------------- correct START to END zero positions	
        for (i = 0; i < nsamp; i++) aTab[permTab[i]] -= corrStart + ((corrEnd - corrStart) * i) / nsamp;



        // ============================================================== SMOOTING TO ZERO
		if (analysisType == ANTYPE_FASTSW)
		{
			// ---------------------------- NO SMOOTING WITH FAST SWEEP
			//for (i = 0; i < nsamp; i++)
			//{
			//	float corr = float(nsamp - i) / nsamp;
			//	aTab[permTab[i]] *= (pow(corr, float(0.5)));
			//}
		}
		else
		{
			for (i = 0; i < nsamp; i++)
			{
				float corr = float(nsamp - i) / nsamp;
				aTab[permTab[i]] *= (corr * corr);
			}
		}
    
        // ============================================================== ALL ZERO TO NN
        for (i = nsamp; i < nn; i++) aTab[permTab[i]] = float(0);
  
    }

	// ======================================================================================
    //   SPECTRUM Samples from circular buffer to aTab
	// ======================================================================================
    else 
    {
		// ---------------------------------------------------------------- CHANNEL 1-2 SELECT using trigCH value (0 or 1)
        j = cbufPos - nn * numCH + trigCH;
        if (j < 0) j += IN_CBUF_LEN;

	    // ---------------------------------------------------------------- ZERO CORRECTION using the sample area values
		static float correction = 0;
		float deltasum = 0;
        int jj = j;
        for (i = 0; i < nn; i++)
        {
            deltasum += (pCbufIn[jj] - correction) * hTab[i]; 
            if ((jj += numCH) >= IN_CBUF_LEN) jj -= IN_CBUF_LEN;
        }
		static float SmoothedCorr;
		SmoothedCorr += (deltasum - SmoothedCorr) * float(0.3);
		correction += SmoothedCorr;

        // ---------------------------------------------------------------- from cBuffer TO aTab, with Corrections and Window
        for (i = 0; i < nn; i++)
        {
            aTab[permTab[i]] = (pCbufIn[j] - correction) * hTab[i];  
            if ((j += numCH) >= IN_CBUF_LEN) j -= IN_CBUF_LEN;
        }
    }


    // ======================================================================================
    //   FHT EXECUTION 
	// ======================================================================================
    le = 1;
    trig_inc = nn >> 1;
    do
    {
        trig_ind = 0;

        j = 0;
        le2 = le * 2;

        do
        {
            ua = cosTab[trig_ind];
            ub = senTab[trig_ind];
            trig_ind += trig_inc;

            i = j;
            mk = le2 - i;

            do
            {
                k = le + i;

                v = ua * A[k] + ub * A[mk];
                
                B[k] = A[i] - v;
                B[i] = A[i] + v;

                mk += le2;
                i += le2;
            }
            while (i < nn);

            j++;
        }
        while (j < le);

        C = A;
        A = B;
        B = C;

        le *= 2;
        trig_inc = trig_inc >> 1;
    }
    while (le < nn);
                          
                           
	// ======================================================================================
    //   Results to responseTab 
	// ======================================================================================
    
    float kb = float((band - 1) / 2.0);
        
    int peakresponse = 1;
    
    if (wavetype == WAVETYPE_NOISE || 
		analysisType == ANTYPE_FASTSW || 
		analysisType == ANTYPE_PULSE) peakresponse = 0;
 
    int index;
    float val;
    if (xlogtype) 
    {
        
        double kf = freqMin * nn / SAMPLES_PER_SEC;
        double kk = log(freqMax / freqMin) / dispx;
        int oldindex = int(kf);
        int bandindex = int(oldindex * band);

        if (peakresponse)
        {
            int n = 0;
            float max = -9;
            for (j = 0; j <= dispx; j++)
            {
                index = int(kf * exp(kk * j));
                if (band != 1) index += int(index * kb);

                if  (index >= bandindex || j == dispx)
                {
                    for (i = oldindex + 1; i <= index; i++)
                    {
                        ua = A[i];
                        ub = A[nn - i];
                        val = sqrt( ua * ua + ub * ub );
                        if (val > max) max = val;
                        n++;
                    }
                    oldindex = index;
                    bandindex = int(oldindex * band);
                }
                
                if (calibratedResponse) max *= calibrateTab[j];
                float newvalue = responseTab[j] += (max - responseTab[j]) * float(responsespeed);
                if (n <= 0) newvalue = -9;
				// This line prevents the NAN or INFINITY numbers from polluting the responseTab
				if (newvalue == newvalue && abs(newvalue) < 30000000) responseTab[j] = newvalue;
                max = -9;               
                n = 0;
            }
        }
        else
        {
            int n = 0;
            float sum = 0;
            for (j = 0; j <= dispx; j++)
            {
                index = int(kf * exp(kk * j));
                if (band != 1) index += int(index * kb);

                if  (index >= bandindex || j == dispx)
                {
                    for (i = oldindex + 1; i <= index; i++)
                    {
                        ua = A[i];
                        ub = A[nn - i];
                        val = sqrt( ua * ua + ub * ub );
						// ----------------------------------------------------- FastSweep correction
						if (analysisType == ANTYPE_FASTSW) val *= float(pow(i, 0.4644));  //POW15=0.4644
						// ----------------------------------------------------- PULSE correction
                        if (analysisType == ANTYPE_PULSE) val *= i;
						// -----------------------------------------------------
                        sum += val;
                        n++;   
                    }
                    oldindex = index;
                    bandindex = int(oldindex * band);
                }
                if (calibratedResponse) sum *= calibrateTab[j];
				float newvalue = responseTab[j] + (sum / n - responseTab[j]) * float(responsespeed);
                if (n <= 0) newvalue = -9;
				// This line prevents the NAN or INFINITY numbers from polluting the responseTab
				if (newvalue == newvalue && abs(newvalue) < 30000000) responseTab[j] = newvalue;
                sum = 0;
                n = 0;
            }
        }

    }
    else
    {
        double kf = double(nn) / SAMPLES_PER_SEC;
        double kk = (freqMax - freqMin) / dispx;
        int oldindex = int(kf * freqMin);
        int bandindex = int(oldindex * band);
        int n = 0;
        float max = -9;
        if (peakresponse)
        {
            for (j = 0; j <= dispx; j++)
            {
                index = int(kf * (freqMin + kk * j));
                if (band != 1) index += int(index * kb);

                if  (index >= bandindex || j == dispx)
                {
                    for (i = oldindex; i < index; i++)
                    {
                        ua = A[i + 1];
                        ub = A[nn - (i + 1)];
                        val = sqrt( ua * ua + ub * ub );
                        if (val > max) max = val; 
                        n++;
                    }
                    oldindex = index;
                    bandindex = int(oldindex * band);
                }
                if (calibratedResponse) max *= calibrateTab[j];
                float newvalue = responseTab[j] + (max - responseTab[j]) * float(responsespeed);
                if (n <= 0) newvalue = -9;
				// This line prevents the NAN or INFINITY numbers from polluting the responseTab
				if (newvalue == newvalue && abs(newvalue) < 30000000) responseTab[j] = newvalue;
                max = -9;
                n = 0;
            }
        }
        else
        {
            int n = 0;
            float sum = 0;
            for (j = 0; j <= dispx; j++)
            {
                index = int(kf * (freqMin + kk * j));
                if (band != 1) index += int(index * kb);

                if  (index >= bandindex || j == dispx)
                { 
                    for (i = oldindex; i < index; i++)
                    {
                        ua = A[i + 1];
                        ub = A[nn - (i + 1)];
                        val = sqrt( ua * ua + ub * ub );
                        // ----------------------------------------------------- FastSweep correction
						if (analysisType == ANTYPE_FASTSW) val *= float(pow(i, 0.4644));  //POW15=0.4644
						// ----------------------------------------------------- PULSE correction
                        if (analysisType == ANTYPE_PULSE) val *= i;
                        // -----------------------------------------------------
						sum += val;
                        n++;
                    }
                    oldindex = index;
                    bandindex = int(oldindex * band);
                }
                if (calibratedResponse) sum *= calibrateTab[j];
                float newvalue = responseTab[j] + (sum / n - responseTab[j]) * float(responsespeed);
                if (n <= 0) newvalue = -9;
				// This line prevents the NAN or INFINITY numbers from polluting the responseTab
				if (newvalue == newvalue && abs(newvalue) < 30000000) responseTab[j] = newvalue;
                sum = 0;
                n = 0;
            }

        }
    }
}


void ResetFhtBuffers()
{
	Sleep(250);
	// ---------------------------------------------------------- reset input buffer
	for (int i = 0; i < IN_CBUF_LEN; i++) pCbufIn[i] = 0;  
	// ---------------------------------------------------------- reset response tab
	for (int i = 0; i < RESP_TAB_LEN; i++) responseTab[i] = 0;
}