#include <afxadv.h> //Has CRecentFileList class definition
//#include "stdafx.h"
//#include "MainFrm.h"
//#include "DaaDoc.h"
//#include "DaaView.h"
//#include "GlobalParams.h"

#include "math.h"

#include "MemoryMappedFiles.h"


// ================================================================================================
// ================================================================================================
//   Theremino Slots - Read Write
// ================================================================================================
// ================================================================================================
double FirstSlot = 0;

MemoryMappedFile MMF1("Theremino1", 4080);

void WriteSlot(double slot, double value)
{
	int s = int(slot);
	if (s < 0 || s > 999) return;
	MMF1.WriteSingle(s * 4, float(value));
}

float ReadSlot(double slot)
{
	int s = int(slot);
	if (s < 0 || s > 999) return 0;
	float v = MMF1.ReadSingle(s * 4);
	if (v != v) v = 0;
	return v;
}




// ================================================================================================
// ================================================================================================
//  TIME FUNCTIONS
// ================================================================================================
// ================================================================================================
static LARGE_INTEGER pfreq, ptime1, ptime2;

void InitTime()
{
    QueryPerformanceFrequency(&pfreq);
    pfreq.QuadPart /= 1000000;  // Time in microseconds
    QueryPerformanceCounter(&ptime1);
}

int GetTime()
{
    QueryPerformanceCounter(&ptime2);
    return (int)((ptime2.QuadPart - ptime1.QuadPart) / pfreq.QuadPart);
}

// you could pass theApp.m_pMainWnd in the "wnd" parameter
void MessageTime(CWnd* wnd)
{
    int t = GetTime() - 15;  // 15 usec overhead
    CString s;
    s.Format("%i", t);
    wnd->MessageBox(s, "", MB_SYSTEMMODAL);
}


// ================================================================================================
// ================================================================================================
//  UTILITY FUNCTIONS - SMOOTING
// ================================================================================================
// ================================================================================================

template <typename T> int sgn(T val) 
{
    return (T(0) < val) - (val < T(0));
}

void SmoothValue_Pow_Adaptive(double& value, double new_value, double speed)
{
	double delta = new_value - value;
	int delta_sgn = sgn(delta);
	double delta_abs = abs(delta);
	double delta_pow = delta_abs * delta_abs * speed;
	if (value != 0) delta_pow /= abs(value);
	if (delta_pow >= delta_abs) value = new_value;
	else value += delta_sgn * delta_pow;
	if (value != value) value = 0; // if not equal to itself then it is a NAN
}

void SmoothValue_Simple(double& value, double new_value, double speed)
{
    value += (new_value - value) * speed;
	if (value != value) value = 0; // if not equal to itself then it is a NAN
}

void SmoothValue(double& value, double new_value, double speed)
{
	return SmoothValue_Pow_Adaptive(value, new_value, speed * 2);
	//return SmoothValue_Simple(value, new_value, speed * 0.01);
}


// ================================================================================================
// ================================================================================================
//   ROUND TO NDIGITS - FLOOR
// ================================================================================================
//   +1.58 = 1.5
//   -1.58 = -1.5
// ================================================================================================
// ================================================================================================ 

double RoundToNdigits_Floor(double n, int nDigits) 
{ 
	// ----------------------------------------------------- can not calculate log(0)
	if (n == 0) return 0;
	// ----------------------------------------------------- calculate shiftAmount
	double shiftAmount = pow(10.0, nDigits);
	// ----------------------------------------------------- shift up
	n *= shiftAmount;
	// ----------------------------------------------------- integer nearest to zero
	if (n >= 0) n = int(floor(n)); else n = -int(floor(abs(n)));
	// ----------------------------------------------------- shift down
	n /= shiftAmount;
	// ----------------------------------------------------- return
	return n;	
} 


// ================================================================================================
// ================================================================================================
//   SMART ROUNDING FUNCTIONS
// ================================================================================================
// ================================================================================================ 

double RoundToNdigits(double n, int nDigits, int nMaxDecimals) 
{ 
	// ----------------------------------------------------- can not calculate log(0)
	if (n == 0) return 0;
	// ----------------------------------------------------- calculate shiftAmount
	int numDP = int(nDigits - floor(log10(abs(n))) - 1);
	if (numDP > nMaxDecimals) numDP = nMaxDecimals;
	double shiftAmount = pow(10.0, numDP);
	// ----------------------------------------------------- shift up
	n *= shiftAmount;
	// ----------------------------------------------------- nearest integer
	if (n > 0) n = int(n + 0.5); else n = int(n - 0.5);
	// ----------------------------------------------------- shift down
	n /= shiftAmount;
	// ----------------------------------------------------- return
	return n;	
} 

double RoundToNdigits2(double n, int nDigits, int nMaxDecimals, double LastSignificativeDigit) 
{ 
	// ----------------------------------------------------- can not calculate log(0)
	if (n == 0) return 0;
	// ----------------------------------------------------- limit to last significative digit
	int magnitude = int(floor(log10(abs(n)))) - 1;
	int magnitudeDiff = magnitude - int(floor(log10(abs(LastSignificativeDigit))));
	if (nDigits > magnitudeDiff) nDigits = magnitudeDiff; 
	// ----------------------------------------------------- calculate shiftAmount
	int numDP = nDigits - magnitude;
	if (numDP > nMaxDecimals) numDP = nMaxDecimals;
	double shiftAmount = pow(10.0, numDP);
	// ----------------------------------------------------- shift up
	n *= shiftAmount;
	// ----------------------------------------------------- nearest integer
	if (n > 0) n = int(n + 0.5); else n = int(n - 0.5);
	// ----------------------------------------------------- shift down
	n /= shiftAmount;
	// ----------------------------------------------------- return
	return n;	
} 


CString MillivoltToString(double mV, int nDigits) 
{
	mV = RoundToNdigits(mV, nDigits, 3);
	CString s;
	if (abs(mV) > 100) s.Format("%g V", mV / 1000);
	else s.Format("%0.0f mV", mV);
	return s;
}

CString MillivoltToString2(double mV, int nDigits) 
{
	mV = RoundToNdigits(mV, nDigits, 3);
	CString s;
	if (abs(mV) > 100) s.Format("%g V", mV / 1000);
	else s.Format("%0.1f mV", mV);
	return s;
}

//CString MillivoltToString3(double mV, int nDigits) 
//{
//	mV = RoundToNdigits(mV, nDigits, 3, 0.1);
//	CString s;
//	if (abs(mV) > 100) s.Format("%g V", mV / 1000);
//	else if (abs(mV) > 0.1) s.Format("%g mV", mV);
//	else if (abs(mV) > 0.0001) s.Format("%g uV", mV * 1000);
//	else s = "0 V";
//	return s;
//}


CString MillisecToString(double mS, int nDigits) 
{
	mS = RoundToNdigits(mS, nDigits, 3);
	CString s;
	if (abs(mS) >= 1000) s.Format("%g Sec", mS / 1000);
	else if (abs(mS) >= 1) s.Format("%g mS", mS);
	else if (abs(mS) >= 0.001) s.Format("%g uS", mS * 1000);
	else s = "0 Sec";
	return s;
}

CString MillisecToString2(double mS, int nDigits) 
{
	mS = RoundToNdigits2(mS, nDigits, 3, 0.000001);
	CString s;
	if (abs(mS) > 100) s.Format("%g Sec", mS / 1000);
	else if (abs(mS) > 0.1) s.Format("%g mS", mS);
	else if (abs(mS) > 0.0001) s.Format("%g uS", mS * 1000);
	else s = "0 Sec";
	return s;
}



CString HertzToString(double Hz, int nDigits) 
{
	//Hz = RoundToNdigits(Hz, nDigits, 1, 0.000001);
	Hz = RoundToNdigits(Hz, nDigits, 2);
	CString s;
	if (Hz > 1000000) s.Format("%g MHz", Hz / 1000000);
	else if (Hz >= 1000) s.Format("%g KHz", Hz / 1000);
	else s.Format("%g Hz", Hz);
	return s;
}

CString HertzToString(double Hz, int nDigits, int nDecimals) 
{
	Hz = RoundToNdigits2(Hz, nDigits, 3, 0.001);
	CString s;
	if (Hz > 1000000) s.Format("%1.3f MHz", Hz / 1000000);
	else if (Hz >= 10000) s.Format("%1.3f KHz", Hz / 1000);
	else if (Hz >= 1000) s.Format("%1.2f KHz", Hz / 1000);
	else
	{
		CString sf;
		sf.Format("%d", nDecimals);
		sf = "%1." + sf + "f Hz";
		s.Format(sf, Hz);
	}
	return s;
}

CString DecibelToString(double dB) 
{
	CString s;
	if (dB >= 10) s.Format(" +%3.1f dB", dB);
	else if (dB >= 0) s.Format(" + %3.1f dB", dB);
	else if (dB > -10) s.Format(" - %3.1f dB", -dB);
	else s.Format(" -%3.1f dB", -dB);
	return s;
}

// ================================================================================================
// ================================================================================================
//   Vpp and dB FUNCTIONS
// ================================================================================================
// ================================================================================================ 
const double Root2x2 = 2.828427124746;

double mVppToDB(double mVpp)
{
	// 0 dB = 1veff = 2.828427 Vpp = 2828.427 mVpp
	return 20 * log10(mVpp / (Root2x2 * 1000));
}

double VrmsTodB(double Vrms)
{
	// 0 dB = 1veff = 2.828427 Vpp = 2828.427 mVpp
	return 20 * log10(Vrms/1000);
}


// ================================================================================================
// ================================================================================================
//   LBUTTON and RBUTTON and KEY PRESSED
// ================================================================================================
// ================================================================================================ 
bool LButtonPressed()
{
	return ((GetAsyncKeyState(VK_LBUTTON) & 0x8000) != 0);
}
bool RButtonPressed()
{
	return ((GetAsyncKeyState(VK_RBUTTON) & 0x8000) != 0);
}
bool KeyPressed(UINT key)
{
	return ((GetAsyncKeyState(key) & 0x8000) != 0);
}


// ================================================================================================
// ================================================================================================
//  DC - PRINT FUNCTIONS
// ================================================================================================
// ================================================================================================
void Print(CDC* pDC, int x, int y, int v)
{
    CString str;
    str.Format(" %i  ", v);
    pDC->TextOut(x, y, str);
}
void Print(CDC* pDC, int x, int y, float v)
{
    CString str;
    str.Format(" %f  ", v);
    pDC->TextOut(x, y, str);
}
void Print(CDC* pDC, int x, int y, double v)
{
    CString str;
    str.Format(" % 10.3f  ", v);
    pDC->TextOut(x, y, str);
}
void Print(CDC* pDC, int x, int y, CString str)
{
    pDC->TextOut(x, y, str);
}


// ================================================================================================
// ================================================================================================
//  DC - Get WindowRect and ClientRect
// ================================================================================================
// ================================================================================================

//CRect GetWrect(CDC* pDC)
//{
//    CWnd* pWnd = pDC->GetWindow();
//    RECT rect;
//    pWnd->GetWindowRect(&rect);
//    return rect;
//}

//CRect GetCrect(CDC* pDC)
//{
//    CWnd* pWnd = pDC->GetWindow();
//    RECT rect;
//    pWnd->GetClientRect(&rect);
//    return rect;
//}


// ================================================================================================
// ================================================================================================
//   COLOR INTERPOLATOR
// ================================================================================================
// ================================================================================================

COLORREF LerpRGB(double index, COLORREF a, COLORREF b)
{
	return RGB(GetRValue(a) + (GetRValue(b) - GetRValue(a)) * index,
			   GetGValue(a) + (GetGValue(b) - GetGValue(a)) * index,
			   GetBValue(a) + (GetBValue(b) - GetBValue(a)) * index);
}

COLORREF InterpolateColor(double index, COLORREF c[], int len)
{
	double d = index * (len-1);
	int i = int(d); 
	if (i > len-1) i = len-1; 
	return LerpRGB(d-i, c[i], c[i+1]);
}


// ================================================================================================
// ================================================================================================
//   MICROMETRIC MOUSE
// ================================================================================================
// ================================================================================================
int CorrectMicrometricMouse(short zDelta)
{
	static int sum = 0;
    sum += zDelta;
    if (abs(sum) < WHEEL_DELTA) return 0;
    int n = sum / WHEEL_DELTA;
	sum = 0;
	return n;
}


// ================================================================================================
// ================================================================================================
//   CORRECT BY CONTROL KEYS
// ================================================================================================
// ================================================================================================
double CorrectByControKeys(double n)
{
	// -----------------------------------------------
	// To get ALT KEY working it has been disabled
	// View CMainFrame::PreTranslateMessage
	// ----------------------------------------------- ALT = / 1000  and ALT + CTRL = / 10000
	if (KeyPressed(VK_MENU))
	{
		if (KeyPressed(VK_CONTROL)) n /= 10000;
		else n /= 1000;
	}
	// ----------------------------------------------- CTRL + SHIFT = / 100
	else if (KeyPressed(VK_CONTROL) && KeyPressed(VK_SHIFT))
    {
		n /= 100;
    }
	// ----------------------------------------------- SHIFT = * 10
    else if (KeyPressed(VK_SHIFT))
    {
		n *= 10;
    }
	// ----------------------------------------------- CTRL = / 10
	else if (KeyPressed(VK_CONTROL))
    {
		n /= 10;
    }
	// -----------------------------------------------
	return n;
}

// ================================================================================================
// ================================================================================================
//   GET / SET - CLIPBOARD TEXT
// ================================================================================================
// ================================================================================================
CString GetTextFromClipBoard(HWND hWnd)
{
	CString strFromClipboard = "";
	// Test to see if we can open the clipboard first
	if (OpenClipboard(hWnd)) 
	{
		// Retrieve the Clipboard data (specifying that 
		// we want ANSI text (via the CF_TEXT value)
		HANDLE hClipboardData = GetClipboardData(CF_TEXT);

		// Call GlobalLock so that to retrieve a pointer
		// to the data associated with the handle returned
		// from GetClipboardData
		char *pchData = (char*)GlobalLock(hClipboardData);

		// Set a local CString variable to the data
		strFromClipboard = pchData;

		// Unlock the global memory
		GlobalUnlock(hClipboardData);

		// Finally, when finished I simply close the Clipboard
		// which has the effect of unlocking it so that other
		// applications can examine or modify its contents
		CloseClipboard();
	}
	return strFromClipboard;
}


void SendTextToClipBoard(HWND hWnd, CString strData)
{
	// test to see if we can open the clipboard first before
	// wasting any cycles with the memory allocation
	if (OpenClipboard(hWnd))
	{
		// Empty the Clipboard. This also has the effect
		// of allowing Windows to free the memory associated
		// with any data that is in the Clipboard
		EmptyClipboard();

		// Ok. We have the Clipboard locked and it's empty
		// Now let's allocate the global memory for our data

		// Here I'm simply using the GlobalAlloc function to 
		// allocate a block of data equal to the text in the
		// "to clipboard" edit control plus one character for the
		// terminating null character required when sending
		// ANSI text to the Clipboard
		HGLOBAL hClipboardData;
		hClipboardData = GlobalAlloc(GMEM_DDESHARE, 
									strData.GetLength()+1);

		// Calling GlobalLock returns to me a pointer to the 
		// data associated with the handle returned from 
		// GlobalAlloc
		char * pchData;
		pchData = (char*)GlobalLock(hClipboardData);
			  
		// At this point, all I need to do is use the standard 
		// C/C++ strcpy function to copy the data from the local 
		// variable to the global memory
		//strcpy(pchData, LPCSTR(strData));
		//strcpy_s(pchData, strData.GetLength(), LPCSTR(strData));

		unsigned int strSize = strData.GetLength();
		memcpy(pchData, strData.GetBuffer(0), strSize * sizeof(TCHAR));
		pchData[strSize] = (TCHAR) 0;//the null terminator
			  
		// Once done, I unlock the memory - remember you 
		// don't call GlobalFree because Windows will free the 
		// memory automatically when EmptyClipboard is next 
		// called
		GlobalUnlock(hClipboardData);
			  
		// Now, set the Clipboard data by specifying that 
		// ANSI text is being used and passing the handle to
		// the global memory
		SetClipboardData(CF_TEXT,hClipboardData);
			  
		// Finally, when finished I simply close the Clipboard
		// which has the effect of unlocking it so that other
		// applications can examine or modify its contents
		CloseClipboard();
	}
}



